@@ -19,29 +19,36 @@
 #ifndef _ABOUTDIALOG_H_
 #define _ABOUTDIALOG_H_
 
-#include <qdialog.h>
+#include <QDialog>
 
 class QLabel;
 class QTextEdit;
-class QPushButton;
+class QDialogButtonBox;
+
+//! Shows the about smplayer dialog
+
+/*!
+ Displays copyright info, license, translators...
+*/
 
 class AboutDialog : public QDialog
 {
 	Q_OBJECT
 
 public:
-	AboutDialog( QWidget * parent = 0, const char * name = 0, 
-                 bool modal = FALSE, WFlags f = 0 );
+	AboutDialog( QWidget * parent = 0, Qt::WindowFlags f = 0 );
 	~AboutDialog();
 
 protected:
+	//! Return a formated string with the translator and language
+	QString trad(const QString & lang, const QString & author);
+
 	QLabel * logo;
 	QLabel * intro;
 	QLabel * foot;
 	QTextEdit * credits;
-	QPushButton * ok_button;
+	QDialogButtonBox * ok_button;
 };
 
-
 #endif
 
