@@ -20,52 +20,48 @@
 #define _BASEGUIPLUS_H_
 
 #include "basegui.h"
-
-
-#if QT_VERSION >= 0x040200
-#define USE_SYSTEM_TRAY
-#else
-#ifdef KDE_SUPPORT
-#define USE_SYSTEM_TRAY
-#endif
-#endif
-
-
-#if QT_VERSION >= 0x040200
 #include <QSystemTrayIcon>
-#include <qpoint.h>
-class Q3PopupMenu;
-#else
-class QPopupMenu;
-class MySystemTrayIcon;
-class QSystemTrayIcon {
-public:
-	enum ActivationReason { Unknown, Context, DoubleClick, Trigger, MiddleClick };
-};
-#endif
+#include <QPoint>
+#include "config.h"
 
+class QMenu;
+class PlaylistDock;
+
+class TimeSliderAction;
+class VolumeSliderAction;
 
 class BaseGuiPlus : public BaseGui
 {
 	Q_OBJECT
 
 public:
-	BaseGuiPlus( QStringList files, QWidget* parent = 0, const char* name = 0, 
-                 WFlags fl = WType_TopLevel );
+	BaseGuiPlus( QWidget* parent = 0, Qt::WindowFlags flags = 0 );
 	~BaseGuiPlus();
 
+	virtual bool startHidden();
+
 protected:
+	virtual void retranslateStrings();
+
 	void loadConfig();
 	void saveConfig();
 	void updateShowAllAct();
+
+    virtual void aboutToEnterFullscreen();
+    virtual void aboutToExitFullscreen();
+	virtual void aboutToEnterCompactMode();
+	virtual void aboutToExitCompactMode();
+
 	virtual void closeEvent( QCloseEvent * e );
 
+	// Functions for other GUI's
+	TimeSliderAction * createTimeSliderAction(QWidget * parent);
+	VolumeSliderAction * createVolumeSliderAction(QWidget * parent);
+
 protected slots:
 	// Reimplemented methods
-	virtual void languageChange();
 	virtual void closeWindow();
 	virtual void setWindowCaption(const QString & title);
-	virtual void firstShow();
 	virtual void resizeWindow(int w, int h);
 	virtual void updateMediaInfo();
 	// New
@@ -74,34 +70,40 @@ protected slots:
 	virtual void showAll(bool b);
 	virtual void quit();
 
+#if DOCK_PLAYLIST
+	virtual void showPlaylist(bool b);
+	void playlistClosed();
+
+	void stretchWindow();
+	void shrinkWindow();
+#endif
 
 protected:
-	#ifdef USE_SYSTEM_TRAY
-
-	#if QT_VERSION >= 0x040200
 	QSystemTrayIcon * tray;
-	Q3PopupMenu * context_menu;
-	#else
-	MySystemTrayIcon * tray;
-	QPopupMenu * context_menu;
-	#endif
+	QMenu * context_menu;
 
 	MyAction * quitAct;
 	MyAction * showTrayAct;
 	MyAction * showAllAct;
 
-	int recents_menu_id;
-
 	// To save state
 	QPoint mainwindow_pos;
 	bool mainwindow_visible;
 
 	QPoint playlist_pos;
-	bool playlist_visible;
+	bool trayicon_playlist_was_visible;
 
 	//QPoint infowindow_pos;
 	//bool infowindow_visible;
-	#endif
+
+#if DOCK_PLAYLIST
+    PlaylistDock * playlistdock;
+	bool fullscreen_playlist_was_visible;
+	bool fullscreen_playlist_was_floating;
+	bool compact_playlist_was_visible;
+	bool ignore_playlist_events;
+#endif
+
 };
 
 #endif
