@@ -26,6 +26,11 @@
 #define SUBTITLES_BY_INDEX 1
 
 
+// Use the new select_sub slave command
+
+#define USE_SELECT_SUB 0
+
+
 // CONTROLWIDGET_OVER_VIDEO
 // if set to 1, the controlwidget will be shown in fullscreen
 // *over* the video (not moving the video) when the user move the mouse 
@@ -44,14 +49,24 @@
 // if 1, the playlist will be docked in the main window, instead
 // of being a top level window
 
-#define DOCK_PLAYLIST 0
+#define DOCK_PLAYLIST 1
 
 
 // STYLE_SWITCHING
 // if 1, the preferences dialog will have an option to switch
 // the Qt style
 
-#define STYLE_SWITCHING 0
+#define STYLE_SWITCHING 1
+
+
+// New code to resize the main window
+
+#define NEW_RESIZE_CODE 1
+
+
+// Allow to use multiple shortcuts for actions
+
+#define USE_MULTIPLE_SHORTCUTS 1
 
 
 // EXTERNAL_SLEEP
@@ -59,11 +74,26 @@
 // instead of QThread::msleep()
 // It can be useful if your Qt doesn't have QThread support.
 // Note: not much test it
+// Note 2: not used anymore
 
 #define EXTERNAL_SLEEP 0
 
 
+// USE_SHORTCUTGETTER
+// if 1, a new dialog will be used to ask the user for a
+// keyshortcut.
+
+#define USE_SHORTCUTGETTER 1
+
+
+// USE_SUBFONT
+// if 1, use -subfont option.
+
+#define USE_SUBFONT 0
+
+
 // Testing with a QGLWidget (for Windows)
 #define USE_GL_WIDGET 0
 
+
 #endif
