@@ -17,9 +17,10 @@
 */
 
 #include "core.h"
-#include <qdir.h>
-#include <qfileinfo.h>
-#include <qregexp.h>
+#include <QDir>
+#include <QFileInfo>
+#include <QRegExp>
+#include <QTextStream>
 
 #include <cmath>
 
@@ -36,21 +37,34 @@
 #ifdef Q_OS_WIN
 /* To change app priority */
 #include <windows.h>
+#include <QSysInfo> // To get Windows version
 #endif
 
 
-Core::Core( MplayerWindow *mpw, QWidget* parent, const char* name ) 
-	: QObject( parent, name ) 
+Core::Core( MplayerWindow *mpw, QWidget* parent ) 
+	: QObject( parent ) 
 {
 	mplayerwindow = mpw;
 
+	_state = Stopped;
+
 	we_are_restarting = false;
-	state = STOPPED;
 	just_loaded_external_subs = false;
 	just_unloaded_external_subs = false;
+	change_volume_after_unpause = false;
+
+	// Create file_settings
+	if (Helper::iniPath().isEmpty()) {
+		file_settings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
+                                      QString(COMPANY), QString("smplayer_files") );
+	} else {
+		QString filename = Helper::iniPath() + "/smplayer_files.ini";
+		file_settings = new QSettings( filename, QSettings::IniFormat );
+		qDebug("Core::Core: file_settings: '%s'", filename.toUtf8().data());
+	}
 
     proc = new MplayerProcess(this);
-    
+
 	connect( proc, SIGNAL(receivedCurrentSec(double)),
              this, SLOT(changeCurrentSec(double)) );
 
@@ -105,13 +119,21 @@ Core::Core( MplayerWindow *mpw, QWidget* parent, const char* name )
 	connect( proc, SIGNAL(receivedStreamTitleAndUrl(QString,QString)),
              this, SLOT(streamTitleAndUrlChanged(QString,QString)) );
 
-	// Mplayerwindow
-	connect( this, SIGNAL(stateChanged(int)),
-             this, SLOT(prepareMplayerWindow(int)) );
+	connect( this, SIGNAL(mediaLoaded()), this, SLOT(autosaveMplayerLog()) );
+	
+	connect( this, SIGNAL(stateChanged(Core::State)), 
+	         this, SLOT(watchState(Core::State)) );
 
 	//pref->load();
 	mset.reset();
 
+	// Mplayerwindow
+	connect( this, SIGNAL(aboutToStartPlaying()),
+             mplayerwindow->videoLayer(), SLOT(playingStarted()) );
+	connect( proc, SIGNAL(processExited()),
+             mplayerwindow->videoLayer(), SLOT(playingStopped()) );
+
+	mplayerwindow->videoLayer()->allowClearingBackground(pref->always_clear_video_background);
 	mplayerwindow->setMonitorAspect( pref->monitor_aspect_double() );
 }
 
@@ -120,10 +142,27 @@ Core::~Core() {
 	saveMediaInfo();
 
     if (proc->isRunning()) stopMplayer();
-    proc->tryTerminate();
+    proc->terminate();
     delete proc;
+	delete file_settings;
 }
 
+void Core::setState(State s) {
+	if (s != _state) {
+		_state = s;
+		emit stateChanged(_state);
+	}
+}
+
+QString Core::stateToString() {
+	if (state()==Playing) return "Playing";
+	else
+	if (state()==Stopped) return "Stopped";
+	else
+	if (state()==Paused) return "Paused";
+	else
+	return "Unknown";
+}
 
 // Public restart
 void Core::restart() {
@@ -136,11 +175,11 @@ void Core::restart() {
 }
 
 bool Core::checkHaveSettingsSaved(QString group_name) {
-	qDebug("Core::checkHaveSettingsSaved: group_name: '%s'", group_name.utf8().data());
+	qDebug("Core::checkHaveSettingsSaved: group_name: '%s'", group_name.toUtf8().data());
 
-	settings->beginGroup( group_name );
-	bool saved = settings->readBoolEntry( "saved", FALSE );
-	settings->endGroup();
+	file_settings->beginGroup( group_name );
+	bool saved = file_settings->value( "saved", false ).toBool();
+	file_settings->endGroup();
 
 	return saved;
 }
@@ -166,31 +205,31 @@ void Core::saveMediaInfo() {
 	}
 
 	if (!group_name.isEmpty()) {
-		settings->beginGroup( group_name );
-		settings->writeEntry( "saved", TRUE);
+		file_settings->beginGroup( group_name );
+		file_settings->setValue( "saved", true);
 
 		/*mdat.save(*settings);*/
-		mset.save();
+		mset.save(file_settings);
 
-		settings->endGroup();
+		file_settings->endGroup();
 	}
 }
 
 void Core::loadMediaInfo(QString group_name) {
-	qDebug("Core::loadMediaInfo: '%s'", group_name.utf8().data() );
+	qDebug("Core::loadMediaInfo: '%s'", group_name.toUtf8().data() );
 
-	settings->beginGroup( group_name );
+	file_settings->beginGroup( group_name );
 
 	/*mdat.load(*settings);*/
-	mset.load();
+	mset.load(file_settings);
 
-	settings->endGroup();
+	file_settings->endGroup();
 }
 
 
 void Core::updateLog(QString line) {
 	if (pref->log_mplayer) {
-		if ( (line.find("A:")==-1) && (line.find("V:")==-1) ) {
+		if ( (line.indexOf("A:")==-1) && (line.indexOf("V:")==-1) ) {
 			mplayer_log += line + "\n";
 		}
 	}
@@ -211,23 +250,23 @@ void Core::updateWidgets() {
 
 
 void Core::tellmp(const QString & command) {
-	qDebug("Core::tellmp: '%s'", command.utf8().data());
+	qDebug("Core::tellmp: '%s'", command.toUtf8().data());
 
-    //qDebug("Command: '%s'", command.utf8().data());
+    //qDebug("Command: '%s'", command.toUtf8().data());
     if (proc->isRunning()) {
-		proc->writeToStdin( command + "\n" );
+		proc->writeToStdin( command );
     } else {
-		qWarning(" tellmp: no process running: %s", command.utf8().data());
+		qWarning(" tellmp: no process running: %s", command.toUtf8().data());
     }
 }
 
 // Generic open, autodetect type
 void Core::open(QString file, int seek) {
-	qDebug("Core::open: '%s'", file.utf8().data());
+	qDebug("Core::open: '%s'", file.toUtf8().data());
 
 	QFileInfo fi(file);
 
-	if ( (fi.exists()) && (fi.extension(false).lower()=="iso") ) {
+	if ( (fi.exists()) && (fi.suffix().toLower()=="iso") ) {
 		qDebug(" * identified as a dvd iso");
 		openDVD("dvd://1:" + file);
 	}
@@ -235,7 +274,7 @@ void Core::open(QString file, int seek) {
 	if ( (fi.exists()) && (!fi.isDir()) ) {
 		qDebug(" * identified as local file");
 		// Local file
-		file = QFileInfo(file).absFilePath();
+		file = QFileInfo(file).absoluteFilePath();
 		openFile(file, seek);
 	} 
 	else
@@ -243,7 +282,7 @@ void Core::open(QString file, int seek) {
 		// Directory
 		qDebug(" * identified as a directory");
 		qDebug("   checking if contains a dvd");
-		file = QFileInfo(file).absFilePath();
+		file = QFileInfo(file).absoluteFilePath();
 		if (Helper::directoryContainsDVD(file)) {
 			qDebug(" * directory contains a dvd");
 			openDVD("dvd://1:"+ file);
@@ -253,13 +292,13 @@ void Core::open(QString file, int seek) {
 		}
 	}
 	else 
-	if (file.lower().startsWith("dvd:")) {
+	if (file.toLower().startsWith("dvd:")) {
 		qDebug(" * identified as dvd");
 		openDVD(file);
 		/*
 		QString f = file.lower();
 		QRegExp s("^dvd://(\\d+)");
-		if (s.search(f) != -1) {
+		if (s.indexIn(f) != -1) {
 			int title = s.cap(1).toInt();
 			openDVD(title);
 		} else {
@@ -269,12 +308,12 @@ void Core::open(QString file, int seek) {
 		*/
 	}
 	else
-	if (file.lower().startsWith("vcd:")) {
+	if (file.toLower().startsWith("vcd:")) {
 		qDebug(" * identified as vcd");
 
-		QString f = file.lower();
+		QString f = file.toLower();
 		QRegExp s("^vcd://(\\d+)");
-		if (s.search(f) != -1) {
+		if (s.indexIn(f) != -1) {
 			int title = s.cap(1).toInt();
 			openVCD(title);
 		} else {
@@ -283,12 +322,12 @@ void Core::open(QString file, int seek) {
 		}
 	}
 	else
-	if (file.lower().startsWith("cdda:")) {
+	if (file.toLower().startsWith("cdda:")) {
 		qDebug(" * identified as cdda");
 
-		QString f = file.lower();
+		QString f = file.toLower();
 		QRegExp s("^cdda://(\\d+)");
-		if (s.search(f) != -1) {
+		if (s.indexIn(f) != -1) {
 			int title = s.cap(1).toInt();
 			openAudioCD(title);
 		} else {
@@ -303,11 +342,11 @@ void Core::open(QString file, int seek) {
 }
 
 void Core::openFile(QString filename, int seek) {
-	qDebug("Core::openFile: '%s'", filename.utf8().data());
+	qDebug("Core::openFile: '%s'", filename.toUtf8().data());
 
 	QFileInfo fi(filename);
 	if (fi.exists()) {
-		playNewFile(fi.absFilePath(), seek);
+		playNewFile(fi.absoluteFilePath(), seek);
 	} else {
 		//File doesn't exists
 		//TODO: error message
@@ -362,7 +401,7 @@ void Core::openDVD( bool from_folder, QString directory) {
 				pref->play_dvd_from_hd = TRUE;
 				openDVD();
 			} else {
-				qDebug("Core::openDVD: directory '%s' is not valid", directory.utf8().data());
+				qDebug("Core::openDVD: directory '%s' is not valid", directory.toUtf8().data());
 			}
 		} else {
 			qDebug("Core::openDVD: directory is empty");
@@ -458,7 +497,7 @@ void Core::openAudioCD(int title) {
 }
 
 void Core::openDVD(QString dvd_url) {
-	qDebug("Core::openDVD: '%s'", dvd_url.utf8().data());
+	qDebug("Core::openDVD: '%s'", dvd_url.toUtf8().data());
 
 	//Checks
 	QString folder = Helper::dvdSplitFolder(dvd_url);
@@ -481,7 +520,7 @@ void Core::openDVD(QString dvd_url) {
 
 	if (proc->isRunning()) {
 		stopMplayer();
-		we_are_restarting = FALSE;
+		we_are_restarting = false;
 	}
 
 	// Save data of previous file:
@@ -503,11 +542,11 @@ void Core::openDVD(QString dvd_url) {
 }
 
 void Core::openStream(QString name) {
-	qDebug("Core::openStream: '%s'", name.utf8().data());
+	qDebug("Core::openStream: '%s'", name.toUtf8().data());
 
 	if (proc->isRunning()) {
 		stopMplayer();
-		we_are_restarting = FALSE;
+		we_are_restarting = false;
 	}
 
 	// Save data of previous file:
@@ -526,11 +565,11 @@ void Core::openStream(QString name) {
 
 
 void Core::playNewFile(QString file, int seek) {
-	qDebug("Core::playNewFile: '%s'", file.utf8().data());
+	qDebug("Core::playNewFile: '%s'", file.toUtf8().data());
 
 	if (proc->isRunning()) {
 		stopMplayer();
-		we_are_restarting = FALSE;
+		we_are_restarting = false;
 	}
 
 	// Save data of previous file:
@@ -565,12 +604,13 @@ void Core::playNewFile(QString file, int seek) {
 
 	/* initializeMenus(); */
 
+	qDebug("Core::playNewFile: volume: %d, old_volume: %d", mset.volume, old_volume);
 	initPlaying(seek);
 }
 
 
 void Core::restartPlay() {
-	we_are_restarting = TRUE;
+	we_are_restarting = true;
 	initPlaying();
 }
 
@@ -617,6 +657,9 @@ void Core::newMediaPlaying() {
 		// Otherwise changeAudio will do nothing.
 
 		int audio = mdat.audios.itemAt(0).ID(); // First one
+		if (mdat.audios.existsItemAt(pref->initial_audio_track-1)) {
+			audio = mdat.audios.itemAt(pref->initial_audio_track-1).ID();
+		}
 
 		// Check if one of the audio tracks is the user preferred.
 		if (!pref->audio_lang.isEmpty()) {
@@ -633,7 +676,7 @@ void Core::newMediaPlaying() {
 #if SUBTITLES_BY_INDEX
 			//Select first subtitle if none selected
 			if (mset.current_sub_id == MediaSettings::NoneSelected) {
-				int sub = mdat.subs.selectOne( pref->subtitle_lang );
+				int sub = mdat.subs.selectOne( pref->subtitle_lang, pref->initial_subtitle_track-1 );
 				changeSubtitle( sub );
 			}
 #else
@@ -690,6 +733,13 @@ void Core::finishRestart() {
 		newMediaPlaying();
 	} 
 
+	if (we_are_restarting) {
+		// Update info about codecs and demuxer
+		mdat.video_codec = proc->mediaData().video_codec;
+		mdat.audio_codec = proc->mediaData().audio_codec;
+		mdat.demuxer = proc->mediaData().demuxer;
+	}
+
 #if SUBTITLES_BY_INDEX
 	//if (we_are_restarting) {
 	if ( (just_loaded_external_subs) || (just_unloaded_external_subs) ) {
@@ -742,7 +792,7 @@ void Core::finishRestart() {
 	}
 #endif
 
-	we_are_restarting = FALSE;
+	we_are_restarting = false;
 
 #if !SUBTITLES_BY_INDEX
 	if (mset.external_subtitles.isEmpty()) {
@@ -755,30 +805,31 @@ void Core::finishRestart() {
 	}
 
 	bool isMuted = mset.mute;
-	setVolume( mset.volume, TRUE );
+	if (!pref->dont_change_volume) setVolume( mset.volume, TRUE );
 	if (isMuted) mute(TRUE);
 
 	setGamma( mset.gamma );
 
 	changePanscan(mset.panscan_factor);
 
-	updateWidgets(); // New
-
 	emit mediaLoaded();
 	emit mediaInfoChanged();
+
+	updateWidgets(); // New
 }
 
 
 void Core::stop()
 {
 	qDebug("Core::stop");
-	qDebug("   state: %d", state);
+	qDebug("   state: %s", stateToString().toUtf8().data());
 	
-	if (state==STOPPED) {
+	if (state()==Stopped) {
 		// if pressed stop twice, reset video to the beginning
 		qDebug("   mset.current_sec: %f", mset.current_sec);
 		mset.current_sec = 0;
-		updateWidgets();
+		emit showTime( mset.current_sec );
+		//updateWidgets();
 	}
 
 	stopMplayer();
@@ -790,11 +841,11 @@ void Core::play()
 {
 	qDebug("Core::play");
     
-	if ((proc->isRunning()) && (state==PAUSED)) {
+	if ((proc->isRunning()) && (state()==Paused)) {
 		tellmp("pause"); // Unpauses
     } 
 	else
-	if ((proc->isRunning()) && (state==PLAYING)) {
+	if ((proc->isRunning()) && (state()==Playing)) {
 		// nothing to do, continue playing
 	}
 	else {
@@ -815,7 +866,7 @@ void Core::pause_and_frame_step() {
 	qDebug("Core::pause_and_frame_step");
 	
 	if (proc->isRunning()) {
-		if (state == PAUSED) {
+		if (state() == Paused) {
 			tellmp("frame_step");
 		}
 		else {
@@ -826,7 +877,7 @@ void Core::pause_and_frame_step() {
 
 void Core::pause() {
 	qDebug("Core::pause");
-	qDebug("Current state: %d", state);
+	qDebug("Current state: %s", stateToString().toUtf8().data());
 
 	if (proc->isRunning()) {
 		// Pauses and unpauses
@@ -856,7 +907,7 @@ void Core::screenshot() {
 	if ( (!pref->screenshot_directory.isEmpty()) && 
          (QFileInfo(pref->screenshot_directory).isDir()) ) 
 	{
-		tellmp("screenshot 0");
+		tellmp("pausing_keep screenshot 0");
 		qDebug(" taken screenshot");
 	} else {
 		qDebug(" error: directory for screenshots not valid");
@@ -881,8 +932,8 @@ void Core::processFinished()
 
 	if (!we_are_restarting) {
 		qDebug("Core::processFinished: play has finished!");
-		state = STOPPED;
-		emit stateChanged(state);
+		setState(Stopped);
+		//emit stateChanged(state());
 	}
 
 	int exit_status = proc->exitStatus();
@@ -908,17 +959,14 @@ void Core::fileReachedEnd() {
 	emit mediaFinished();
 }
 
-void Core::goToPos(int perc)
-{
-    qDebug("Core::goToPos: per: %d", perc);
-
-    tellmp ( "seek " + QString::number( perc) + " 1");
+void Core::goToPos(int perc) {
+	qDebug("Core::goToPos: per: %d", perc);
+	tellmp( "seek " + QString::number(perc) + " 1");
 }
 
 
 
-void Core::startMplayer( QString file, double seek )
-{
+void Core::startMplayer( QString file, double seek ) {
 	qDebug("Core::startMplayer");
 
 	if (file.isEmpty()) {
@@ -937,7 +985,7 @@ void Core::startMplayer( QString file, double seek )
 	}
 
 	mplayer_log = "";
-	bool is_mkv = (QFileInfo(file).extension(FALSE).lower() == "mkv");
+	bool is_mkv = (QFileInfo(file).suffix().toLower() == "mkv");
 
 	// DVD
 	QString dvd_folder;
@@ -945,11 +993,23 @@ void Core::startMplayer( QString file, double seek )
 	if (mdat.type==TYPE_DVD) {
 		dvd_folder = Helper::dvdSplitFolder(file);
 		if (dvd_folder.isEmpty()) dvd_folder = pref->dvd_device;
-		if (dvd_folder.endsWith("/")) dvd_folder = dvd_folder.remove( dvd_folder.length()-1,1);
+		// Remove trailing "/"
+		if (dvd_folder.endsWith("/")) {
+#ifdef Q_OS_WIN
+			QRegExp r("^[A-Z]:/$");
+			int pos = r.indexIn(dvd_folder);
+			qDebug("Core::startMplayer: drive check: '%s': regexp: %d", dvd_folder.toUtf8().data(), pos);
+			if (pos == -1)
+#endif
+				dvd_folder = dvd_folder.remove( dvd_folder.length()-1, 1);
+		}
 		dvd_title = Helper::dvdSplitTitle(file);
 		file = "dvd://" + QString::number(dvd_title);
 	}
 
+	// URL
+	bool url_is_playlist = file.endsWith(IS_PLAYLIST_TAG);
+	if (url_is_playlist) file = file.remove( QRegExp(IS_PLAYLIST_TAG_RX) );
 
 	proc->clearArguments();
 
@@ -957,12 +1017,37 @@ void Core::startMplayer( QString file, double seek )
 	if ( (!pref->screenshot_directory.isEmpty()) && 
          (QFileInfo(pref->screenshot_directory).isDir()) ) 
 	{
-		qDebug("Core::startMplayer: setting working directory to '%s'", pref->screenshot_directory.utf8().data());
-		proc->setWorkingDirectory( QDir( pref->screenshot_directory ) );
+		qDebug("Core::startMplayer: setting working directory to '%s'", pref->screenshot_directory.toUtf8().data());
+		proc->setWorkingDirectory( pref->screenshot_directory );
 	}
 
+	// Use absolute path
+	QString mplayer_bin = pref->mplayer_bin;
+	QFileInfo fi(mplayer_bin);
+    if (fi.exists()) {
+        mplayer_bin = fi.absoluteFilePath();
+	}
 
-	proc->addArgument( pref->mplayer_bin );
+/*
+#ifdef Q_OS_WIN
+	// Windows 98 and ME: call another program as intermediate
+	if ( (QSysInfo::WindowsVersion == QSysInfo::WV_98) ||
+         (QSysInfo::WindowsVersion == QSysInfo::WV_Me) ) 
+	{
+		QString intermediate_bin = Helper::mplayer_intermediate(mplayer_bin);
+		if (!intermediate_bin.isEmpty()) {
+			proc->addArgument( intermediate_bin );
+		}
+	}
+#endif
+*/
+
+	proc->addArgument( mplayer_bin );
+
+	/*
+	proc->addArgument("-key-fifo-size");
+	proc->addArgument("1000");
+	*/
 
 	proc->addArgument("-noquiet");
 
@@ -988,6 +1073,21 @@ void Core::startMplayer( QString file, double seek )
 		proc->addArgument("hwac3");
 	}
 
+	if ( (pref->h264_skip_loop) ||
+         (pref->h264_skip_frames) )
+	{
+		proc->addArgument("-lavdopts");
+		QString o;
+		if (pref->h264_skip_loop) {
+			o = "skiploopfilter=all";
+		}
+		if (pref->h264_skip_frames) {
+			if (!o.isEmpty()) o += ":";
+			o += "skipframe=nonref";
+		}
+		proc->addArgument(o);
+	}
+
 	proc->addArgument("-sub-fuzziness");
 #if SUBTITLES_BY_INDEX
 	proc->addArgument( QString::number(pref->subfuzziness) );
@@ -1075,6 +1175,13 @@ void Core::startMplayer( QString file, double seek )
 		proc->addArgument( QString::number( pref->autosync_factor ) );
 	}
 
+	if (pref->use_direct_rendering) {
+		proc->addArgument("-dr");
+	}
+
+	if (!pref->use_double_buffer) {
+		proc->addArgument("-nodouble");
+	}
 
 #ifndef Q_OS_WIN
 	if (!pref->use_mplayer_window) {
@@ -1086,12 +1193,14 @@ void Core::startMplayer( QString file, double seek )
 #ifndef Q_OS_WIN
 	if (pref->disable_screensaver) {
 		proc->addArgument("-stop-xscreensaver");
+	} else {
+		proc->addArgument("-nostop-xscreensaver");
 	}
 #endif
 
 	if (!pref->use_mplayer_window) {
 		proc->addArgument("-wid");
-		proc->addArgument( QString::number( (int) mplayerwindow->mplayerLayer()->winId() ) );
+		proc->addArgument( QString::number( (int) mplayerwindow->videoLayer()->winId() ) );
 	
 		proc->addArgument("-colorkey");
 		//proc->addArgument( QString::number(COLORKEY) );
@@ -1132,16 +1241,19 @@ void Core::startMplayer( QString file, double seek )
 		proc->addArgument("-font");
 		proc->addArgument( pref->font_file );
 
+#if USE_SUBFONT
 		if (pref->use_subfont) {
 			proc->addArgument("-subfont");
 			proc->addArgument( pref->font_file );
 		}
+#endif
 	}
 
 	proc->addArgument( "-subfont-autoscale");
 	proc->addArgument( QString::number( pref->font_autoscale ) );
 	proc->addArgument( "-subfont-text-scale");
-	proc->addArgument( QString::number( pref->font_textscale ) );
+	//proc->addArgument( QString::number( pref->font_textscale ) );
+	proc->addArgument( QString::number(mset.sub_scale) );
 
 	if (!pref->subcp.isEmpty()) {
 		proc->addArgument("-subcp");
@@ -1154,11 +1266,11 @@ void Core::startMplayer( QString file, double seek )
 	}
 
 	if (!mset.external_subtitles.isEmpty()) {
-		if (QFileInfo(mset.external_subtitles).extension(FALSE).lower()=="idx") {
+		if (QFileInfo(mset.external_subtitles).suffix().toLower()=="idx") {
 			// sub/idx subtitles
 			QFileInfo fi(mset.external_subtitles);
-			QString s = fi.dirPath() +"/"+ fi.baseName(FALSE);
-			qDebug(" * subtitle file without extension: '%s'", s.utf8().data());
+			QString s = fi.path() +"/"+ fi.baseName();
+			qDebug(" * subtitle file without extension: '%s'", s.toUtf8().data());
 			proc->addArgument("-vobsub");
 			proc->addArgument( s );
 		} else {
@@ -1281,6 +1393,14 @@ void Core::startMplayer( QString file, double seek )
 	proc->addArgument("-osdlevel");
 	proc->addArgument( QString::number( pref->osd ) );
 
+	if (mset.flip) {
+		proc->addArgument("-flip");
+	}
+
+	if (pref->use_idx) {
+		proc->addArgument("-idx");
+	}
+
 	// Video filters:
 	// Phase
 	if (mset.phase_filter) {
@@ -1341,7 +1461,7 @@ void Core::startMplayer( QString file, double seek )
 	}
 
 	// Postprocessing
-	if ( (mset.postprocessing_filter) || (pref->use_postprocessing) ) {
+	if (mset.postprocessing_filter) {
 		proc->addArgument("-vf-add");
 		proc->addArgument("pp");
 		proc->addArgument("-autoq");
@@ -1373,7 +1493,9 @@ void Core::startMplayer( QString file, double seek )
 	}
 
 	// Screenshot
-	if (!pref->screenshot_directory.isEmpty()) {
+	if ( (!pref->screenshot_directory.isEmpty()) && 
+        (QFileInfo(pref->screenshot_directory).isDir()) )
+	{
 		// Subtitles on screenshots
 		if (pref->subtitles_on_screenshots) {
 			if (pref->use_ass_subtitles) {
@@ -1417,11 +1539,16 @@ void Core::startMplayer( QString file, double seek )
 		af += "extrastereo";
 	}
 
-	if ( (pref->use_volnorm) || (mset.volnorm_filter) ) {
+	if (mset.volnorm_filter) {
 		if (!af.isEmpty()) af += ",";
 		af += "volnorm=2";
 	}
 
+	if (pref->use_scaletempo) {
+		if (!af.isEmpty()) af += ",";
+		af += "scaletempo";
+	}
+
 	// Additional audio filters, supplied by user
 	// File
 	if ( !pref->mplayer_additional_audio_filters.isEmpty() ) {
@@ -1450,12 +1577,10 @@ void Core::startMplayer( QString file, double seek )
 		proc->addArgument("0");
 	}
 
-	proc->addArgument( file );
-
 	// Additional options supplied by the user
 	// File
 	if (!mset.mplayer_additional_options.isEmpty()) {
-		QStringList args = QStringList::split(" ", mset.mplayer_additional_options);
+		QStringList args = mset.mplayer_additional_options.split(" ");
         QStringList::Iterator it = args.begin();
         while( it != args.end() ) {
  			proc->addArgument( (*it) );
@@ -1464,7 +1589,7 @@ void Core::startMplayer( QString file, double seek )
 	}
 	// Global
 	if (!pref->mplayer_additional_options.isEmpty()) {
-		QStringList args = QStringList::split(" ", pref->mplayer_additional_options);
+		QStringList args = pref->mplayer_additional_options.split(" ");
         QStringList::Iterator it = args.begin();
         while( it != args.end() ) {
  			proc->addArgument( (*it) );
@@ -1472,8 +1597,16 @@ void Core::startMplayer( QString file, double seek )
 		}
 	}
 
+	// File to play
+	if (url_is_playlist) {
+		proc->addArgument("-playlist");
+	}
+
+	proc->addArgument( file );
+
 	//Log command
 	//mplayer_log = "Command: \n";
+	/*
 	QString commandline;
     QStringList list = proc->arguments();
     QStringList::Iterator it = list.begin();
@@ -1482,9 +1615,12 @@ void Core::startMplayer( QString file, double seek )
 		commandline += " ";
         ++it;
     }
+	*/
+	QString commandline = proc->arguments().join(" ");
 	mplayer_log += commandline + "\n\n";
-	qDebug("Core::startMplayer: command: '%s'", commandline.utf8().data());
+	qDebug("Core::startMplayer: command: '%s'", commandline.toUtf8().data());
 
+	emit aboutToStartPlaying();
 	
 	if ( !proc->start() ) {
 	    // error handling
@@ -1494,7 +1630,7 @@ void Core::startMplayer( QString file, double seek )
 	//stopped_by_user = FALSE;
 
 	// Try to set the volume as soon as possible
-	//tellmp("volume " + QString::number(mset.volume) + " 1");
+	tellmp("volume " + QString::number(mset.volume) + " 1");
 }
 
 void Core::stopMplayer() {
@@ -1508,7 +1644,10 @@ void Core::stopMplayer() {
     tellmp("quit");
     
 	qDebug("Core::stopMplayer: Waiting mplayer to finish...");
-	Helper::finishProcess( proc );
+	//Helper::finishProcess( proc );
+	if (!proc->waitForFinished(5000)) {
+		proc->kill();
+	}
 
 	qDebug("Core::stopMplayer: Finished. (I hope)");
 }
@@ -1581,7 +1720,9 @@ void Core::wheelUp() {
 	switch (pref->wheel_function) {
 		case Preferences::Volume : incVolume(); break;
 		case Preferences::Zoom : incPanscan(); break;
-		default : forward( pref->seeking4 );
+		case Preferences::Seeking : forward( pref->seeking4 ); break;
+		case Preferences::ChangeSpeed : incSpeed(); break;
+		default : {} // do nothing
 	}
 }
 
@@ -1590,7 +1731,9 @@ void Core::wheelDown() {
 	switch (pref->wheel_function) {
 		case Preferences::Volume : decVolume(); break;
 		case Preferences::Zoom : decPanscan(); break;
-		default : rewind( pref->seeking4 );
+		case Preferences::Seeking : rewind( pref->seeking4 ); break;
+		case Preferences::ChangeSpeed : decSpeed(); break;
+		default : {} // do nothing
 	}
 }
 
@@ -1604,7 +1747,22 @@ void Core::toggleRepeat(bool b) {
 	qDebug("Core::toggleRepeat: %d", b);
 	if ( pref->loop != b ) {
 		pref->loop = b;
-		restartPlay();
+		if (proc->isRunning()) restartPlay();
+	}
+}
+
+
+void Core::toggleFlip() {
+	qDebug("Core::toggleFlip");
+	toggleFlip( !mset.flip );
+}
+
+void Core::toggleFlip(bool b) {
+	qDebug("Core::toggleFlip: %d", b);
+
+	if (mset.flip != b) {
+		mset.flip = b;
+		if (proc->isRunning()) restartPlay();
 	}
 }
 
@@ -1873,13 +2031,18 @@ void Core::setVolume(int volume, bool force) {
 	if ((volume==mset.volume) && (!force)) return;
 
 	mset.volume = volume;
-	if (mset.volume > 99 ) mset.volume = 99;
+	if (mset.volume > 100 ) mset.volume = 100;
 	if (mset.volume < 0 ) mset.volume = 0;
 
-    tellmp("volume " + QString::number(volume) + " 1");
+	if (state() == Paused) {
+		// Change volume later, after quiting pause
+		change_volume_after_unpause = true;
+	} else {
+		tellmp("volume " + QString::number(volume) + " 1");
+	}
 
 	//if (mset.mute) mute(TRUE);
-	mset.mute=FALSE;
+	mset.mute=false;
 
 	updateWidgets();
 
@@ -1901,7 +2064,7 @@ void Core::mute(bool b) {
 
 	int v = 0;
 	if (mset.mute) v = 1;
-	tellmp("mute " + QString::number(v) );
+	tellmp("pausing_keep mute " + QString::number(v) );
 
 	updateWidgets();
 }
@@ -1960,6 +2123,38 @@ void Core::decSubPos() {
 	tellmp("sub_pos " + QString::number( mset.sub_pos ) + " 1");
 }
 
+void Core::changeSubScale(int value) {
+	qDebug("Core::changeSubScale: %d", value);
+
+	bool need_restart = false;
+	if (pref->use_ass_subtitles || 
+        pref->change_sub_scale_should_restart == Preferences::Enabled)
+	{
+		need_restart = true;
+	}
+	else
+	if (pref->change_sub_scale_should_restart == Preferences::Detect) {
+		need_restart = (!proc->isMplayerAtLeast(23745));
+	}
+
+	if (value < 0) value = 0;
+	if (value != mset.sub_scale) {
+		mset.sub_scale = value;
+		if (need_restart) {
+			restartPlay();
+		} else {
+			tellmp("sub_scale " + QString::number( mset.sub_scale ) + " 1");
+		}
+	}
+}
+
+void Core::incSubScale() {
+	changeSubScale( mset.sub_scale + 1 );
+}
+
+void Core::decSubScale() {
+	changeSubScale( mset.sub_scale - 1 );
+}
 
 void Core::incSubStep() {
 	qDebug("Core::incSubStep");
@@ -1979,11 +2174,11 @@ void Core::changeCurrentSec(double sec) {
 		mset.current_sec -= mset.starting_time;
 	}
 	
-	if (state != PLAYING) {
-		state = PLAYING;
+	if (state() != Playing) {
+		setState(Playing);
 		qDebug("mplayer reports that now it's playing");
 		emit mediaStartPlay();
-		emit stateChanged(state);
+		//emit stateChanged(state());
 	}
 
 	emit showTime(mset.current_sec);
@@ -2002,8 +2197,8 @@ void Core::gotStartingTime(double time) {
 void Core::changePause() {
 	qDebug("Core::changePause");
 	qDebug("mplayer reports that it's paused");
-	state = PAUSED;
-	emit stateChanged(state);
+	setState(Paused);
+	//emit stateChanged(state());
 }
 
 void Core::changeDeinterlace(int ID) {
@@ -2032,7 +2227,30 @@ void Core::changeSubtitle(int ID) {
 	}
 	
 	qDebug("Core::changeSubtitle: ID: %d", ID);
+#if USE_SELECT_SUB && SUBTITLES_BY_INDEX
+	int real_id = -1;
+	if (ID == -1) {
+		tellmp( "sub_source -1" );
+	} else {
+		real_id = mdat.subs.itemAt(ID).ID();
+		switch (mdat.subs.itemAt(ID).type()) {
+			case SubData::Vob:
+				tellmp( "sub_vob " + QString::number(real_id) );
+				break;
+			case SubData::Sub:
+				tellmp( "sub_demux " + QString::number(real_id) );
+				break;
+			case SubData::File:
+				tellmp( "sub_file " + QString::number(real_id) );
+				break;
+			default: {
+				qWarning("Core::changeSubtitle: unknown type!");
+			}
+		}
+	}
+#else
 	tellmp( "sub_select " + QString::number(ID) );
+#endif
 	updateWidgets();
 }
 
@@ -2086,12 +2304,22 @@ void Core::changeAudio(int ID) {
 	if (ID!=mset.current_audio_id) {
 		mset.current_audio_id = ID;
 		qDebug("changeAudio: ID: %d", ID);
-		
-		if (pref->audio_change_requires_restart) {
+
+		bool need_restart = (pref->fast_audio_change == Preferences::Disabled);
+		if (pref->fast_audio_change == Preferences::Detect) {
+			need_restart = (!proc->isMplayerAtLeast(21441));
+		}
+
+		if (need_restart) {
 			restartPlay(); 
 		} else {
 			tellmp("switch_audio " + QString::number(ID) );
-			if (mset.mute) mute(TRUE); // if muted, mute again
+			#ifdef Q_OS_WIN
+			// Workaround for a mplayer problem in windows,
+			// volume is too loud after changing audio.
+			setVolume( mset.volume, true );
+			#endif
+			if (mset.mute) mute(true); // if muted, mute again
 			updateWidgets();
 		}
 	}
@@ -2249,7 +2477,7 @@ void Core::changeAspectRatio( int ID ) {
 		mset.panscan_filter = QString("scale=%1:%2,").arg(real_width).arg(mdat.video_height);
 		mset.panscan_filter += QString("crop=%1:%2").arg(round(mdat.video_height * 4 /3)).arg(mdat.video_height);
 		//mset.crop = QSize( mdat.video_height * 4 /3, mdat.video_height );
-		qDebug(" panscan_filter = '%s'", mset.panscan_filter.utf8().data() );
+		qDebug(" panscan_filter = '%s'", mset.panscan_filter.toUtf8().data() );
 
 	}
     else
@@ -2272,7 +2500,7 @@ void Core::changeAspectRatio( int ID ) {
 			asp = (double) 16 / 9;
 			mset.crop_43to169_filter = QString("scale=%1:%2,").arg(real_width).arg(mdat.video_height);
 			mset.crop_43to169_filter += QString("crop=%1:%2").arg(real_width).arg(height);
-			qDebug(" crop_43to169_filter = '%s'", mset.crop_43to169_filter.utf8().data() );
+			qDebug(" crop_43to169_filter = '%s'", mset.crop_43to169_filter.toUtf8().data() );
 		}
 	}
 	else
@@ -2287,7 +2515,10 @@ void Core::changeAspectRatio( int ID ) {
 		mset.crop_43to169_filter = "";
         switch (ID) {
         	//case MediaSettings::AspectAuto: asp = mdat.video_aspect; break;
-			case MediaSettings::AspectAuto: asp = mset.win_aspect(); break;
+			case MediaSettings::AspectAuto: {
+				qDebug("Core::changeAspectRatio: mset.win_width %d, mset.win_height: %d", mset.win_width, mset.win_height);
+                asp = mset.win_aspect(); break;
+			}
             case MediaSettings::Aspect43: asp = (double) 4 / 3; break;
             case MediaSettings::Aspect169: asp = (double) 16 / 9; break;
 			case MediaSettings::Aspect149: asp = (double) 14 / 9; break;
@@ -2297,8 +2528,12 @@ void Core::changeAspectRatio( int ID ) {
 		}
 	}
 
-	mplayerwindow->setAspect( asp );
-    //tellmp("switch_ratio " + QString::number( asp ) );
+	if (!pref->use_mplayer_window) {
+		mplayerwindow->setAspect( asp );
+	} else {
+		// Using mplayer own window
+		tellmp("switch_ratio " + QString::number(asp));
+	}
 
 	updateWidgets();
 
@@ -2363,6 +2598,14 @@ void Core::decPanscan() {
 	changePanscan( mset.panscan_factor - 0.10 );
 }
 
+void Core::changeUseAss(bool b) {
+	qDebug("Core::changeUseAss: %d", b);
+
+	if (pref->use_ass_subtitles != b) {
+		pref->use_ass_subtitles = b;
+		if (proc->isRunning()) restartPlay();
+	}
+}
 
 void Core::displayMessage(QString text) {
 	qDebug("Core::displayMessage");
@@ -2373,7 +2616,13 @@ void Core::displayScreenshotName(QString filename) {
 	qDebug("Core::displayScreenshotName");
 	//QString text = tr("Screenshot saved as %1").arg(filename);
 	QString text = QString("Screenshot saved as %1").arg(filename);
-	tellmp("osd_show_text \"" + text + "\" 3000 1");
+
+	if (state() != Paused) {
+		// Dont' show the message on OSD while in pause, otherwise
+		// the video goes forward a frame.
+		tellmp("pausing_keep osd_show_text \"" + text + "\" 3000 1");
+	}
+
 	emit showMessage(text);
 }
 
@@ -2417,7 +2666,7 @@ void Core::gotNoVideo() {
 }
 
 void Core::gotVO(QString vo) {
-	qDebug("Core::gotVO: '%s'", vo.utf8().data() );
+	qDebug("Core::gotVO: '%s'", vo.toUtf8().data() );
 
 	if ( pref->vo.isEmpty()) {
 		qDebug("saving vo");
@@ -2426,7 +2675,7 @@ void Core::gotVO(QString vo) {
 }
 
 void Core::gotAO(QString ao) {
-	qDebug("Core::gotAO: '%s'", ao.utf8().data() );
+	qDebug("Core::gotAO: '%s'", ao.toUtf8().data() );
 
 	if ( pref->ao.isEmpty()) {
 		qDebug("saving ao");
@@ -2434,26 +2683,42 @@ void Core::gotAO(QString ao) {
 	}
 }
 
-void Core::prepareMplayerWindow() {
-	prepareMplayerWindow(state);
-}
-
-void Core::prepareMplayerWindow(int state) {
-	qDebug("Core::prepareMplayerWindow: state: %d", state);
-
-	if (pref->always_clear_background) {
-		mplayerwindow->allowClearingBackground(true);
-	} else {
-		if (state == PLAYING)
-			mplayerwindow->allowClearingBackground(false);
-		else
-		if (state == STOPPED) 
-			mplayerwindow->allowClearingBackground(true);
-	}
-}
-
 void Core::streamTitleAndUrlChanged(QString title, QString url) {
 	mdat.stream_title = title;
 	mdat.stream_url = url;
 	emit mediaInfoChanged();
 }
+
+/*! 
+	Save the mplayer log to a file, so it can be used by external
+	applications.
+*/
+void Core::autosaveMplayerLog() {
+	qDebug("Core::autosaveMplayerLog");
+
+    //mplayer log autosaving
+    if (pref->autosave_mplayer_log) {
+        if (!pref->mplayer_log_saveto.isEmpty()) {
+            QFile file( pref->mplayer_log_saveto );
+            if ( file.open( QIODevice::WriteOnly ) ) {
+                QTextStream strm( &file );
+                strm << mplayer_log;
+                file.close();
+            }
+        }
+    }
+    //mplayer log autosaving end
+}
+
+//!  Called when the state changes
+void Core::watchState(Core::State state) {
+	if ((state == Playing) && (change_volume_after_unpause)) 
+	{
+		// Delayed volume change
+		qDebug("Core::watchState: delayed volume change");
+		tellmp("volume " + QString::number(mset.volume) + " 1");
+		change_volume_after_unpause = false;
+	}
+}
+
+#include "moc_core.cpp"
