@@ -20,7 +20,7 @@
 #ifndef _DESKTOPINFO_H_
 #define _DESKTOPINFO_H_
 
-#include <qsize.h>
+#include <QSize>
 
 class QWidget;
 
