@@ -19,15 +19,15 @@
 #ifndef _ENCODINGS_H_
 #define _ENCODINGS_H_
 
-#include <qobject.h>
-#include <qstringlist.h>
+#include <QObject>
+#include <QStringList>
 
 class Encodings : public QObject 
 {
 	Q_OBJECT
 
 public:
-	Encodings( QObject * parent = 0, const char * name = 0 );
+	Encodings( QObject * parent = 0 );
 	~Encodings();
 
 	QStringList list() { return l;} ;
