@@ -17,20 +17,22 @@
 */
 
 #include "eqslider.h"
-#include <qslider.h>
-#include <qlabel.h>
-#include <qpixmap.h>
+#include <QSlider>
+#include <QLabel>
+#include <QPixmap>
 #include "verticaltext.h"
-#include "qt3_4_compat.h"
 
-EqSlider::EqSlider( QWidget* parent, const char* name, WFlags fl) 
-	: EqSliderBase(parent, name, fl)
+
+EqSlider::EqSlider( QWidget* parent, Qt::WindowFlags f) 
+	: QWidget(parent, f)
 {
+	setupUi(this);
+
 	_icon->setText( QString::null );
-	_slider->setFocusPolicy( STRONGFOCUS );
-	_slider->setTickmarks( QSlider::Right );
+	_slider->setFocusPolicy( Qt::StrongFocus );
+	_slider->setTickPosition( QSlider::TicksRight );
 	_slider->setTickInterval( 10 );
-	_slider->setLineStep( 1 );
+	_slider->setSingleStep( 1 );
 	_slider->setPageStep( 10 );
 
 	connect( _slider, SIGNAL(valueChanged(int)),
@@ -40,9 +42,10 @@ EqSlider::EqSlider( QWidget* parent, const char* name, WFlags fl)
 EqSlider::~EqSlider() {
 }
 
-
+/*
 void EqSlider::languageChange() {
 }
+*/
 
 void EqSlider::setIcon( QPixmap i) {
 	_icon->setPixmap(i);
@@ -61,28 +64,15 @@ QString EqSlider::label() const {
 }
 
 void EqSlider::setValue(int value) {
-    #if QT_VERSION >= 0x040000
 	_slider->setValue(value);
-	#else
-	// QSlider in Qt 3works upside-down!
-	_slider->setValue(-value);
-	#endif
 }
 
 int EqSlider::value() const {
-    #if QT_VERSION >= 0x040000
 	return _slider->value();
-	#else
-	// QSlider in Qt 3 works upside-down!
-	return - _slider->value();
-	#endif
 }
 
 void EqSlider::sliderValueChanged(int v) {
-    #if QT_VERSION >= 0x040000
 	emit valueChanged( v );
-	#else
-	// QSlider in Qt 3 works upside-down!
-	emit valueChanged( -v );
-	#endif
 }
+
+#include "moc_eqslider.cpp"
