@@ -19,10 +19,12 @@
 
 #include "global.h"
 #include "constants.h"
-#include <qsettings.h>
+#include <QSettings>
 #include "preferences.h"
 #include "translator.h"
-#include <qapplication.h>
+#include "helper.h"
+#include <QApplication>
+#include <QFile>
 
 QSettings * settings;
 Preferences * pref;
@@ -33,25 +35,41 @@ void global_init(const QString & ini_path) {
 	qDebug("global_init");
 
 	// Translator
-	translator = new Translator;
+	translator = new Translator();
 
 	// settings
-#if QT_VERSION >= 0x040000
-	if (!ini_path.isEmpty()) {
-		settings = new QSettings( ini_path + "/smplayer.ini",
-                                  QSettings::IniFormat );
+	Helper::setIniPath(ini_path);
+	if (Helper::iniPath().isEmpty()) {
+		settings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
+    	                         QString(COMPANY), QString(PROGRAM) );
+	} else {
+		QString filename = Helper::iniPath() + "/smplayer.ini";
+		settings = new QSettings( filename, QSettings::IniFormat );
+		qDebug("global_init: config file: '%s'", filename.toUtf8().data());
+
 	}
-	else {
+
+	/*
+	if (!ini_path.isEmpty()) {
+		QString file = ini_path + "/smplayer.ini";
+		settings = new QSettings( file, QSettings::IniFormat );
+		qDebug("global_init: config file: '%s'", file.toUtf8().data());
+	}
+	else 
+	if (QFile::exists(Helper::appHomePath())) {
+		QString file = Helper::appHomePath() + "/smplayer.ini";
+		settings = new QSettings( file, QSettings::IniFormat );
+		qDebug("global_init: config file: '%s'", file.toUtf8().data());
+	}
+	else
+	{
 		settings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
     	                         QString(COMPANY), QString(PROGRAM) );
 	}
-#else
-	settings = new QSettings(QSettings::Ini);
-	settings->beginGroup( "/" + QString(PROGRAM) +"/" );
-#endif
+	*/
 
 	// Preferences
-	pref = new Preferences;
+	pref = new Preferences();
 }
 
 void global_end() {
@@ -61,11 +79,6 @@ void global_end() {
 	delete pref;
 	pref = 0;
 
-	// settings
-	#if QT_VERSION < 0x040000
-	settings->endGroup();
-	#endif
-
 	delete settings;
 	delete translator;
 }
