@@ -20,7 +20,7 @@
 #ifndef _GLOBAL_H_
 #define _GLOBAL_H_
 
-#include <qstring.h>
+#include <QString>
 
 // Some global objects
 
