@@ -19,16 +19,27 @@
 #ifndef _IMAGES_H_
 #define _IMAGES_H_
 
-#include <qpixmap.h>
-#include <qstring.h>
+#include <QPixmap>
+#include <QString>
 
 /* Warning: don't use this until global->preferences is created! */
 class Images
 {
 
 public:
-	static QPixmap icon(QString name, int size=-1, bool png=TRUE);
+	static QPixmap icon(QString name, int size=-1, bool png = true);
+	static QPixmap flippedIcon(QString name, int size=-1, bool png = true);
+
 	static QPixmap resize(QPixmap *p, int size=20);
+	static QPixmap flip(QPixmap *p);
+
+private:
+	//! Return the filename for the icon
+	static QString filename(const QString & name, bool png);
+
+	//! Try to load an icon. \a icon_name is the filename of the
+	//! icon without path. Return a null pixmap if loads fails.
+	static QPixmap loadIcon(const QString & icon_name);
 };
 
 #endif
