@@ -20,15 +20,15 @@
 #define _INFOFILE_H_
 
 #include "mediadata.h"
-#include <qstring.h>
-#include <qobject.h>
+#include <QString>
+#include <QObject>
 
 class InfoFile : public QObject
 {
 	Q_OBJECT
 
 public:
-	InfoFile( QObject * parent = 0, const char * name = 0 );
+	InfoFile( QObject * parent = 0 );
 	~InfoFile();
 
 	QString getInfo(MediaData md);
