@@ -19,21 +19,21 @@
 #ifndef _INPUTDVDDIRECTORY_H_
 #define _INPUTDVDDIRECTORY_H_
 
-#include "inputdvddirectorybase.h"
+#include "ui_inputdvddirectory.h"
 
-class InputDVDDirectory : public InputDVDDirectoryBase
+class InputDVDDirectory : public QDialog, public Ui::InputDVDDirectory
 {
 	Q_OBJECT
 
 public:
-	InputDVDDirectory( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
+	InputDVDDirectory( QWidget* parent = 0, Qt::WindowFlags f = 0 );
 	~InputDVDDirectory();
 
 	void setFolder(QString folder);
 	QString folder();
 
 protected slots:
-    virtual void selectDVDFolder();
+    void on_searchButton_clicked();
 };
 
 
