@@ -19,23 +19,26 @@
 #ifndef _LOGWINDOW_H_
 #define _LOGWINDOW_H_
 
-#include "logwindowbase.h"
+#include "ui_logwindowbase.h"
 
-class LogWindow : public LogWindowBase
+class LogWindow : public QWidget, public Ui::LogWindowBase
 {
 	Q_OBJECT
 
 public:
-	LogWindow( QWidget* parent = 0, const char* name = 0);
+	LogWindow( QWidget* parent = 0);
 	~LogWindow();
 
 	void setText(QString log);
 	QString text();
 
+protected:
+	virtual void retranslateStrings();
+	virtual void changeEvent ( QEvent * event ) ;
+
 protected slots:
-	virtual void languageChange();
-    virtual void copyText();
-    virtual void save();
+	void on_copyButton_clicked();
+    void on_saveButton_clicked();
 };
 
 
