@@ -17,8 +17,7 @@
 */
 
 #include "mediadata.h"
-#include <qsettings.h>
-#include <qfileinfo.h>
+#include <QFileInfo>
 #include <cmath>
 
 
@@ -37,11 +36,6 @@ void MediaData::reset() {
 
 	novideo = FALSE;
 
-	// Some defaults in case file is only audio...
-    /*
-	video_width=200;
-    video_height=150;
-	*/
 	video_width=0;
     video_height=0;
     video_aspect= (double) 4/3;
@@ -106,7 +100,7 @@ QString MediaData::displayName() {
 void MediaData::list() {
 	qDebug("MediaData::list");
 
-	qDebug("  filename: '%s'", filename.utf8().data());
+	qDebug("  filename: '%s'", filename.toUtf8().data());
 	qDebug("  duration: %f", duration);
 
 	qDebug("  video_width: %d", video_width); 
@@ -115,7 +109,7 @@ void MediaData::list() {
 
 	qDebug("  type: %d", type);
 	qDebug("  novideo: %d", novideo);
-	qDebug("  dvd_id: '%s'", dvd_id.utf8().data());
+	qDebug("  dvd_id: '%s'", dvd_id.toUtf8().data());
 
 	qDebug("  initialized: %d", initialized);
 
@@ -137,15 +131,15 @@ void MediaData::list() {
 	//qDebug("  chapters: %d", chapters);
 	//qDebug("  angles: %d", angles);
 
-	qDebug("  demuxer: '%s'", demuxer.utf8().data() );
-	qDebug("  video_format: '%s'", video_format.utf8().data() );
-	qDebug("  audio_format: '%s'", audio_format.utf8().data() );
+	qDebug("  demuxer: '%s'", demuxer.toUtf8().data() );
+	qDebug("  video_format: '%s'", video_format.toUtf8().data() );
+	qDebug("  audio_format: '%s'", audio_format.toUtf8().data() );
 	qDebug("  video_bitrate: %d", video_bitrate );
-	qDebug("  video_fps: '%s'", video_fps.utf8().data() );
+	qDebug("  video_fps: '%s'", video_fps.toUtf8().data() );
 	qDebug("  audio_bitrate: %d", audio_bitrate );
 	qDebug("  audio_rate: %d", audio_rate );
 	qDebug("  audio_nch: %d", audio_nch );
-	qDebug("  video_codec: '%s'", video_codec.utf8().data() );
-	qDebug("  audio_codec: '%s'", audio_codec.utf8().data() );
+	qDebug("  video_codec: '%s'", video_codec.toUtf8().data() );
+	qDebug("  audio_codec: '%s'", audio_codec.toUtf8().data() );
 }
 
