@@ -17,41 +17,29 @@
 */
 
 #include "mplayerprocess.h"
-#include <qregexp.h>
-#include <qstringlist.h>
-#include <qapplication.h>
+#include <QRegExp>
+#include <QStringList>
+#include <QApplication>
+
 #include "global.h"
 #include "preferences.h"
 #include "config.h"
 
-#if DONT_USE_SIGNALS
-#include <qtimer.h>
-#endif
 
-MplayerProcess::MplayerProcess(QObject * parent, const char * name)
- : QProcess(parent,name) 
+MplayerProcess::MplayerProcess(QObject * parent) : MyProcess(parent) 
 {
-	setCommunication( QProcess::Stdin | QProcess::Stdout | QProcess::Stderr );
-#if DONT_USE_SIGNALS
-	timer = new QTimer( this );
-	connect( timer, SIGNAL(timeout()), this, SLOT(read()) );
-#else
-	connect( this, SIGNAL(readyReadStdout()),
-             this, SLOT(read()) );
-#endif
-	connect( this, SIGNAL(readyReadStderr()),
-             this, SLOT(readFromStderr()) );
+	connect( this, SIGNAL(lineAvailable(QByteArray)),
+			 this, SLOT(parseLine(QByteArray)) );
 
+	connect( this, SIGNAL(finished(int,QProcess::ExitStatus)), 
+             this, SLOT(processFinished()) );
 
-	connect( this, SIGNAL(processExited()),
-             this, SLOT(stopReading()) );
+	connect( this, SIGNAL(error(QProcess::ProcessError)),
+             this, SLOT(gotError(QProcess::ProcessError)) );
 
-	connect( this, SIGNAL(lineAvailable(QString)),
-			 this, SLOT(parseLine(QString)) );
-
-	incomplete_line = "";
-	notified_mplayer_is_running = FALSE;
+	notified_mplayer_is_running = false;
 	last_sub_id = -1;
+	mplayer_svn = -1; // Not found yet
 
 	init_rx();
 }
@@ -59,90 +47,50 @@ MplayerProcess::MplayerProcess(QObject * parent, const char * name)
 MplayerProcess::~MplayerProcess() {
 }
 
-bool MplayerProcess::start( QStringList * env ) {
+bool MplayerProcess::isMplayerAtLeast(int svn_revision) {
+	qDebug("MplayerProcess::isMplayerAtLeast: comparing %d with %d", svn_revision, mplayer_svn);
+
+	if (mplayer_svn == -1) {
+		qWarning("MplayerProcess::isMplayerAtLeast: no version found!");
+	}
+	else
+	if (mplayer_svn == 0) {
+		qWarning("MplayerProcess::isMplayerAtLeast: version couldn't be parsed!");
+	}
+
+	if (mplayer_svn <= 0) {
+		qWarning("MplayerProcess::isMplayerAtLeast: assuming that the mplayer version is greater than %d", svn_revision);
+		return true;
+	}
+
+	return (mplayer_svn >= svn_revision);
+}
+
+bool MplayerProcess::start() {
 	init_rx(); // Update configurable regular expressions
 
 	md.reset();
-	notified_mplayer_is_running = FALSE;
-	incomplete_line = "";
+	notified_mplayer_is_running = false;
 	last_sub_id = -1;
+	mplayer_svn = -1; // Not found yet
+	received_end_of_file = false;
 
-#if !DONT_USE_SIGNALS
-	return QProcess::start(env);
-#else
-	int r = QProcess::start(env);
-	QTimer * t = new QTimer(this);
-	connect( t, SIGNAL(timeout()), this, SLOT(startReading()) );
-	t->start(100, TRUE);
-	return r;
-#endif
+	MyProcess::start();
+	return waitForStarted();
 }
 
-void MplayerProcess::startReading() {
-#if DONT_USE_SIGNALS
-	qDebug("MplayerProcess::startReading");
-	timer->start(40);
-#endif
-}
-
-void MplayerProcess::stopReading() {
-#if DONT_USE_SIGNALS
-	qDebug("MplayerProcess::stopReading");
-	timer->stop();
-#endif
-}
-
-void MplayerProcess::readFromStderr() {
-	QString line;
-	while (canReadLineStderr()) {
-		line = "stderr: " + readLineStderr();
-		emit lineAvailable(line);
+void MplayerProcess::writeToStdin(QString text) {
+	if (isRunning()) {
+		//qDebug("MplayerProcess::writeToStdin");
+		write( text.toLocal8Bit() + "\n");
+	} else {
+		qWarning("MplayerProcess::writeToStdin: process not running");
 	}
 }
 
-#if DONT_USE_SIGNALS
-void MplayerProcess::read() {
-	//qDebug("MplayerProcess::read");
-	QByteArray ba;
-	ba = readStdout();
-	if ( !ba.isEmpty() ) {
-		read( ba );
-	}
-}
-#else
-void MplayerProcess::read() {
-	//qDebug("MplayerProcess::read");
-	read( readStdout() );
-}
-#endif
-
-void MplayerProcess::read(const QByteArray & new_text) {
-	QString line;
-	QString l = incomplete_line;
-	l += new_text;
-
-	l = l.replace(0x0D, '\n');
-
-	//int n=0;
-	int pos = l.find('\n');
-	while (pos > -1) {
-		line = l.left(pos);
-		emit lineAvailable( QString::fromLocal8Bit(line) );
-		l = l.mid(pos+1);
-		#ifdef Q_OS_WIN
-		// If line starts with \n, remove it
-		if (l.startsWith("\n")) l = l.mid(1);
-		#endif
-		pos = l.find('\n');
-		/*
-		n++;
-		if (n > 10) { n=0; qApp->processEvents(); }
-		*/
-	} 
-	incomplete_line = l;
-	//qDebug("incomplete_line: '%s'", incomplete_line.utf8().data());
-}
-
+//static QRegExp rx_mplayer_revision("^MPlayer (\\S+)-SVN-r(\\d+)-(.*)");
+static QRegExp rx_mplayer_revision("^MPlayer (.*)-r(\\d+)(.*)");
+static QRegExp rx_mplayer_version("^MPlayer ([a-z,0-9,.]+)-(.*)");
 static QRegExp rx_av("^[AV]: *([0-9,:.-]+)");
 static QRegExp rx_frame("^[AV]:.* (\\d+)\\/.\\d+");// [0-9,.]+");
 static QRegExp rx("^(.*)=(.*)");
@@ -179,16 +127,16 @@ static QRegExp rx_subs("^ID_SID_(\\d+)_(LANG|NAME)=(.*)");
 #endif
 
 //Clip info
-static QRegExp rx_clip_name("^ (name|title): (.*)", false);
-static QRegExp rx_clip_artist("^ artist: (.*)", false);
-static QRegExp rx_clip_author("^ author: (.*)", false);
-static QRegExp rx_clip_album("^ album: (.*)", false);
-static QRegExp rx_clip_genre("^ genre: (.*)", false);
-static QRegExp rx_clip_date("^ (creation date|year): (.*)", false);
-static QRegExp rx_clip_track("^ track: (.*)", false);
-static QRegExp rx_clip_copyright("^ copyright: (.*)", false);
-static QRegExp rx_clip_comment("^ comment: (.*)", false);
-static QRegExp rx_clip_software("^ software: (.*)", false);
+static QRegExp rx_clip_name("^ (name|title): (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_artist("^ artist: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_author("^ author: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_album("^ album: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_genre("^ genre: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_date("^ (creation date|year): (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_track("^ track: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_copyright("^ copyright: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_comment("^ comment: (.*)", Qt::CaseInsensitive);
+static QRegExp rx_clip_software("^ software: (.*)", Qt::CaseInsensitive);
 
 static QRegExp rx_stream_title("^.* StreamTitle='(.*)';StreamUrl='(.*)';");
 
@@ -203,48 +151,53 @@ void MplayerProcess::init_rx() {
 }
 
 
-void MplayerProcess::parseLine(QString line) {
-	//qDebug("MplayerProcess::parseLine: '%s'", line.utf8().data() );
+void MplayerProcess::parseLine(QByteArray ba) {
+	//qDebug("MplayerProcess::parseLine: '%s'", ba.data() );
 
 	QString tag;
 	QString value;
 
 	// Parse A: V: line
-	//qDebug("%s", line.utf8().data());
-	if (rx_av.search(line) > -1) {
+	//qDebug("%s", line.toUtf8().data());
+	if (rx_av.indexIn(ba) > -1) {
 		double sec = rx_av.cap(1).toDouble();
-		//qDebug("cap(1): '%s'", rx_av.cap(1).utf8().data() );
+		//qDebug("cap(1): '%s'", rx_av.cap(1).toUtf8().data() );
 		//qDebug("sec: %f", sec);
+		
 		if (!notified_mplayer_is_running) {
 			qDebug("MplayerProcess::parseLine: starting sec: %f", sec);
 			emit receivedStartingTime(sec);
 			emit mplayerFullyLoaded();
-			notified_mplayer_is_running = TRUE;
+			notified_mplayer_is_running = true;
 		}
-
+		
 	    emit receivedCurrentSec( sec );
 
 		// Check for frame
-		if (rx_frame.search(line) > -1) {
+		if (rx_frame.indexIn(ba) > -1) {
 			int frame = rx_frame.cap(1).toInt();
 			//qDebug(" frame: %d", frame);
 			emit receivedCurrentFrame(frame);
 		}
 	}
 	else {
+		QString line = QString::fromLocal8Bit( ba );
+		//QString line = QString::fromUtf8( ba );
+		emit lineAvailable(line);
+
 		// Parse other things
-		qDebug("MplayerProcess::parseLine: '%s'", line.utf8().data() );
+		qDebug("MplayerProcess::parseLine: '%s'", line.toUtf8().data() );
 
 		// Screenshot
-		if (rx_screenshot.search(line) > -1) {
+		if (rx_screenshot.indexIn(line) > -1) {
 			QString shot = rx_screenshot.cap(1);
-			qDebug("MplayerProcess::parseLine: screenshot: '%s'", shot.utf8().data());
+			qDebug("MplayerProcess::parseLine: screenshot: '%s'", shot.toUtf8().data());
 			emit receivedScreenshot( shot );
 		}
 		else
 
 		// End of file
-		if (rx_endoffile.search(line) > -1) {
+		if (rx_endoffile.indexIn(line) > -1) {
 			qDebug("MplayerProcess::parseLine: detected end of file");
 
 			// In case of playing VCDs or DVDs, maybe the first title
@@ -256,12 +209,14 @@ void MplayerProcess::parseLine(QString line) {
 				emit mplayerFullyLoaded();
 			}
 
-			emit receivedEndOfFile();
+			//emit receivedEndOfFile();
+			// Send signal once the process is finished, not now!
+			received_end_of_file = true;
 		}
 		else
 
 		// Window resolution
-		if (rx_winresolution.search(line) > -1) {
+		if (rx_winresolution.indexIn(line) > -1) {
 			/*
 			md.win_width = rx_winresolution.cap(4).toInt();
 			md.win_height = rx_winresolution.cap(5).toInt();
@@ -278,7 +233,7 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// No video
-		if (rx_novideo.search(line) > -1) {
+		if (rx_novideo.indexIn(line) > -1) {
 			md.novideo = TRUE;
 			emit receivedNoVideo();
 			//emit mplayerFullyLoaded();
@@ -286,16 +241,16 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// Pause
-		if (rx_paused.search(line) > -1) {
+		if (rx_paused.indexIn(line) > -1) {
 			emit receivedPause();
 		}
 
 		// Stream title
-		if (rx_stream_title.search(line) > -1) {
+		if (rx_stream_title.indexIn(line) > -1) {
 			QString s = rx_stream_title.cap(1);
 			QString url = rx_stream_title.cap(2);
-			qDebug("MplayerProcess::parseLine: stream_title: '%s'", s.utf8().data());
-			qDebug("MplayerProcess::parseLine: stream_url: '%s'", url.utf8().data());
+			qDebug("MplayerProcess::parseLine: stream_title: '%s'", s.toUtf8().data());
+			qDebug("MplayerProcess::parseLine: stream_url: '%s'", url.toUtf8().data());
 			md.stream_title = s;
 			md.stream_url = url;
 			emit receivedStreamTitleAndUrl( s, url );
@@ -308,34 +263,53 @@ void MplayerProcess::parseLine(QString line) {
 			return;
 		}
 
+		if (mplayer_svn == -1) {
+			if (rx_mplayer_revision.indexIn(line) > -1) {
+				mplayer_svn = rx_mplayer_revision.cap(2).toInt();
+				qDebug("MplayerProcess::parseLine: MPlayer SVN revision found: %d", mplayer_svn);
+			} 
+			else
+			if (rx_mplayer_version.indexIn(line) > -1) {
+				QString version = rx_mplayer_version.cap(1);
+				qDebug("MplayerProcess::parseLine: MPlayer version found: %s", version.toUtf8().data());
+				mplayer_svn = 0;
+				if (version == "1.0rc2") mplayer_svn = 24722;
+				else
+				if (version == "1.0rc1") mplayer_svn = 20372;
+				else qWarning("MplayerProcess::parseLine: unknown MPlayer version");
+
+				qDebug("MplayerProcess::parseLine: MPlayer SVN: %d", mplayer_svn);
+			}
+		}
+
 #if SUBTITLES_BY_INDEX
-		if (rx_subtitle.search(line) > -1) {
+		if (rx_subtitle.indexIn(line) > -1) {
 			md.subs.process(line);
 		}
 		else
-		if (rx_sid.search(line) > -1) {
+		if (rx_sid.indexIn(line) > -1) {
 			md.subs.process(line);
 		}
 		else
-		if (rx_subtitle_file.search(line) > -1) {
+		if (rx_subtitle_file.indexIn(line) > -1) {
 			md.subs.process(line);
 		}
 #endif
 
 		// AO
-		if (rx_ao.search(line) > -1) {
+		if (rx_ao.indexIn(line) > -1) {
 			emit receivedAO( rx_ao.cap(1) );
 		}
 		else
 
 #if !SUBTITLES_BY_INDEX
 		// Matroska subtitles
-		if (rx_subs.search(line) > -1) {
+		if (rx_subs.indexIn(line) > -1) {
 			int ID = rx_subs.cap(1).toInt();
 			QString lang = rx_subs.cap(3);
 			QString t = rx_subs.cap(2);
 			qDebug("MplayerProcess::parseLine: Subs: ID: %d, Lang: '%s' Type: '%s'", 
-                    ID, lang.utf8().data(), t.utf8().data());
+                    ID, lang.toUtf8().data(), t.toUtf8().data());
 
 			if ( t == "NAME" )
 				md.subtitles.addName(ID, lang);
@@ -346,12 +320,12 @@ void MplayerProcess::parseLine(QString line) {
 #endif
 
 		// Matroska audio
-		if (rx_audio_mat.search(line) > -1) {
+		if (rx_audio_mat.indexIn(line) > -1) {
 			int ID = rx_audio_mat.cap(1).toInt();
 			QString lang = rx_audio_mat.cap(3);
 			QString t = rx_audio_mat.cap(2);
 			qDebug("MplayerProcess::parseLine: Audio: ID: %d, Lang: '%s' Type: '%s'", 
-                    ID, lang.utf8().data(), t.utf8().data());
+                    ID, lang.toUtf8().data(), t.toUtf8().data());
 
 			if ( t == "NAME" ) 
 				md.audios.addName(ID, lang);
@@ -361,7 +335,7 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// Matroshka chapters
-		if (rx_mkvchapters.search(line)!=-1) {
+		if (rx_mkvchapters.indexIn(line)!=-1) {
 			int c = rx_mkvchapters.cap(1).toInt();
 			qDebug("MplayerProcess::parseLine: mkv chapters: %d", c);
 			if (c > md.mkv_chapters) {
@@ -372,7 +346,7 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// VCD titles
-		if (rx_vcd.search(line) > -1 ) {
+		if (rx_vcd.indexIn(line) > -1 ) {
 			int ID = rx_vcd.cap(1).toInt();
 			QString length = rx_vcd.cap(2);
 			//md.titles.addID( ID );
@@ -381,12 +355,12 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// Audio CD titles
-		if (rx_cdda.search(line) > -1 ) {
+		if (rx_cdda.indexIn(line) > -1 ) {
 			int ID = rx_cdda.cap(1).toInt();
 			QString length = rx_cdda.cap(2);
 			double duration = 0;
 			QRegExp r("(\\d+):(\\d+):(\\d+)");
-			if ( r.search(length) > -1 ) {
+			if ( r.indexIn(length) > -1 ) {
 				duration = r.cap(1).toInt() * 60;
 				duration += r.cap(2).toInt();
 			}
@@ -400,7 +374,7 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// DVD titles
-		if (rx_title.search(line) > -1) {
+		if (rx_title.indexIn(line) > -1) {
 			int ID = rx_title.cap(1).toInt();
 			QString t = rx_title.cap(2);
 
@@ -425,31 +399,31 @@ void MplayerProcess::parseLine(QString line) {
 		else
 
 		// Catch cache messages
-		if (rx_cache.search(line) > -1) {
+		if (rx_cache.indexIn(line) > -1) {
 			emit receivedCacheMessage(line);
 		}
 		else
 
 		// Creating index
-		if (rx_create_index.search(line) > -1) {
+		if (rx_create_index.indexIn(line) > -1) {
 			emit receivedCreatingIndex(line);
 		}
 		else
 
 		// Catch connecting message
-		if (rx_connecting.search(line) > -1) {
+		if (rx_connecting.indexIn(line) > -1) {
 			emit receivedConnectingToMessage(line);
 		}
 		else
 
 		// Catch resolving message
-		if (rx_resolving.search(line) > -1) {
+		if (rx_resolving.indexIn(line) > -1) {
 			emit receivedResolvingMessage(line);
 		}
 		else
 
 		// Aspect ratio for old versions of mplayer
-		if (rx_aspect2.search(line) > -1) {
+		if (rx_aspect2.indexIn(line) > -1) {
 			md.video_aspect = rx_aspect2.cap(1).toDouble();
 			qDebug("MplayerProcess::parseLine: md.video_aspect set to %f", md.video_aspect);
 		}
@@ -457,98 +431,98 @@ void MplayerProcess::parseLine(QString line) {
 
 		// Clip info
 		// Name
-		if (rx_clip_name.search(line) > -1) {
+		if (rx_clip_name.indexIn(line) > -1) {
 			QString s = rx_clip_name.cap(2);
-			qDebug("MplayerProcess::parseLine: clip_name: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_name: '%s'", s.toUtf8().data());
 			md.clip_name = s;
 		}
 		else
 
 		// Artist
-		if (rx_clip_artist.search(line) > -1) {
+		if (rx_clip_artist.indexIn(line) > -1) {
 			QString s = rx_clip_artist.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_artist: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_artist: '%s'", s.toUtf8().data());
 			md.clip_artist = s;
 		}
 		else
 
 		// Author
-		if (rx_clip_author.search(line) > -1) {
+		if (rx_clip_author.indexIn(line) > -1) {
 			QString s = rx_clip_author.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_author: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_author: '%s'", s.toUtf8().data());
 			md.clip_author = s;
 		}
 		else
 
 		// Album
-		if (rx_clip_album.search(line) > -1) {
+		if (rx_clip_album.indexIn(line) > -1) {
 			QString s = rx_clip_album.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_album: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_album: '%s'", s.toUtf8().data());
 			md.clip_album = s;
 		}
 		else
 
 		// Genre
-		if (rx_clip_genre.search(line) > -1) {
+		if (rx_clip_genre.indexIn(line) > -1) {
 			QString s = rx_clip_genre.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_genre: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_genre: '%s'", s.toUtf8().data());
 			md.clip_genre = s;
 		}
 		else
 
 		// Date
-		if (rx_clip_date.search(line) > -1) {
+		if (rx_clip_date.indexIn(line) > -1) {
 			QString s = rx_clip_date.cap(2);
-			qDebug("MplayerProcess::parseLine: clip_date: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_date: '%s'", s.toUtf8().data());
 			md.clip_date = s;
 		}
 		else
 
 		// Track
-		if (rx_clip_track.search(line) > -1) {
+		if (rx_clip_track.indexIn(line) > -1) {
 			QString s = rx_clip_track.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_track: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_track: '%s'", s.toUtf8().data());
 			md.clip_track = s;
 		}
 		else
 
 		// Copyright
-		if (rx_clip_copyright.search(line) > -1) {
+		if (rx_clip_copyright.indexIn(line) > -1) {
 			QString s = rx_clip_copyright.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_copyright: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_copyright: '%s'", s.toUtf8().data());
 			md.clip_copyright = s;
 		}
 		else
 
 		// Comment
-		if (rx_clip_comment.search(line) > -1) {
+		if (rx_clip_comment.indexIn(line) > -1) {
 			QString s = rx_clip_comment.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_comment: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_comment: '%s'", s.toUtf8().data());
 			md.clip_comment = s;
 		}
 		else
 
 		// Software
-		if (rx_clip_software.search(line) > -1) {
+		if (rx_clip_software.indexIn(line) > -1) {
 			QString s = rx_clip_software.cap(1);
-			qDebug("MplayerProcess::parseLine: clip_software: '%s'", s.utf8().data());
+			qDebug("MplayerProcess::parseLine: clip_software: '%s'", s.toUtf8().data());
 			md.clip_software = s;
 		}
 		else
 
 		// Catch starting message
 		/*
-		pos = rx_play.search(line);
+		pos = rx_play.indexIn(line);
 		if (pos > -1) {
 			emit mplayerFullyLoaded();
 		}
 		*/
 
 		//Generic things
-		if (rx.search(line) > -1) {
+		if (rx.indexIn(line) > -1) {
 			tag = rx.cap(1);
 			value = rx.cap(2);
-			//qDebug("MplayerProcess::parseLine: tag: %s, value: %s", tag.utf8().data(), value.utf8().data());
+			//qDebug("MplayerProcess::parseLine: tag: %s, value: %s", tag.toUtf8().data(), value.toUtf8().data());
 
 			// Generic audio
 			if (tag == "ID_AUDIO_ID") {
@@ -579,7 +553,7 @@ void MplayerProcess::parseLine(QString line) {
 			
 			if (tag == "ID_FILE_SUB_FILENAME") {
 				QString name = value;
-				qDebug("MplayerProcess::parseLine: SUB_FILENAME: %s", name.utf8().data() );
+				qDebug("MplayerProcess::parseLine: SUB_FILENAME: %s", name.toUtf8().data() );
 				if (last_sub_id != -1) md.subtitles.addFilename( last_sub_id, name );
 				/*
 				if (!md.subtitles.existsFilename(name)) {
@@ -617,7 +591,7 @@ void MplayerProcess::parseLine(QString line) {
 			else
 			if (tag == "ID_DVD_DISC_ID") {
 				md.dvd_id = value;
-				qDebug("MplayerProcess::parseLine: md.dvd_id set to '%s'", md.dvd_id.utf8().data());
+				qDebug("MplayerProcess::parseLine: md.dvd_id set to '%s'", md.dvd_id.toUtf8().data());
 			}
 			else
 			if (tag == "ID_DEMUXER") {
@@ -661,5 +635,20 @@ void MplayerProcess::parseLine(QString line) {
 			}
 		}
 	}
-
 }
+
+// Called when the process is finished
+void MplayerProcess::processFinished() {
+	qDebug("MplayerProcess::processFinished");
+	// Send this signal before the endoffile one, otherwise
+	// the playlist will start to play next file before all
+	// objects are notified that the process has exited.
+	emit processExited();
+	if (received_end_of_file) emit receivedEndOfFile();
+}
+
+void MplayerProcess::gotError(QProcess::ProcessError error) {
+	qDebug("MplayerProcess::gotError: %d", (int) error);
+}
+
+#include "moc_mplayerprocess.cpp"
