@@ -19,38 +19,30 @@
 #ifndef _MPLAYERPROCESS_H_
 #define _MPLAYERPROCESS_H_
 
-#include <qprocess.h>
-#include <qstring.h>
-//#include <qcstring.h>
+#include <QString>
+#include "myprocess.h"
 #include "mediadata.h"
 
-
-#ifdef Q_OS_WIN
-#define DONT_USE_SIGNALS 1
-#else
-#define DONT_USE_SIGNALS 0
-#endif
-
-
-#if DONT_USE_SIGNALS
-class QTimer;
-#endif
-
 class QStringList;
 
-class MplayerProcess : public QProcess 
+class MplayerProcess : public MyProcess 
 {
 	Q_OBJECT
 
 public:
-	MplayerProcess(QObject * parent = 0, const char * name = 0 );
+	MplayerProcess(QObject * parent = 0);
 	~MplayerProcess();
 
-	virtual bool start( QStringList * env = 0 );
+	bool start();
+	void writeToStdin(QString text);
 
 	MediaData mediaData() { return md; };
 
+	int mplayerVersion() { return mplayer_svn; };
+	bool isMplayerAtLeast(int svn_revision);
+
 signals:
+	void processExited();
 	void lineAvailable(QString line);
 
 	void receivedCurrentSec(double sec);
@@ -72,30 +64,23 @@ signals:
 
 	void receivedStreamTitleAndUrl(QString,QString);
 
-
 protected slots:
-	void read();
-	void read(const QByteArray & new_text);
-	void readFromStderr();
-	void parseLine(QString line);
-
-	void startReading();
-	void stopReading();
+	void parseLine(QByteArray ba);
+	void processFinished();
+	void gotError(QProcess::ProcessError);
 
 protected:
 	void init_rx();
 
 private:
-	QString incomplete_line;
 	bool notified_mplayer_is_running;
+	bool received_end_of_file;
 
 	MediaData md;
 
 	int last_sub_id;
 
-#if DONT_USE_SIGNALS
-	QTimer * timer;
-#endif
+	int mplayer_svn;
 };
 
 
