@@ -20,9 +20,16 @@
 #ifndef MPLAYERWINDOW_H
 #define MPLAYERWINDOW_H
 
-#include <qwidget.h>
-#include <qsize.h>
-#include <qpoint.h>
+#include <QWidget>
+#include <QSize>
+#include <QPoint>
+
+#include <QResizeEvent>
+#include <QWheelEvent>
+#include <QMouseEvent>
+#include <QKeyEvent>
+#include <QPaintEvent>
+
 #include "config.h"
 
 class QWidget;
@@ -30,20 +37,20 @@ class QLabel;
 class QKeyEvent;
 
 #if USE_GL_WIDGET
-#include <qgl.h>
+#include <QGLWidget>
 #define SC_WIDGET QGLWidget
 #else
 #define SC_WIDGET QWidget
 #endif
 
-/* This class hides the mouse cursor after some seconds, if not moved */
+//! Screen is a widget that hides the mouse cursor after some seconds if not moved.
 
 class Screen : public SC_WIDGET
 {
 	Q_OBJECT
 
 public:
-	Screen(QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
+	Screen(QWidget* parent = 0, Qt::WindowFlags f = 0);
 	~Screen();
 
 signals:
@@ -60,24 +67,36 @@ private:
 	QPoint cursor_pos, last_cursor_pos;
 };
 
-/* This class ca be set to not delete the background */
+//! MplayerLayer can be instructed to not delete the background.
 
 class MplayerLayer : public Screen
 {
 	Q_OBJECT
 
 public:
-	MplayerLayer(QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
+	MplayerLayer(QWidget* parent = 0, Qt::WindowFlags f = 0);
 	~MplayerLayer();
 
+	//! If b is true, the background of the widget will be repainted as usual.
+	/*! Otherwise the background will not repainted when a video is playing. */
 	void allowClearingBackground(bool b);
+
+	//! Return true if repainting the background is allowed.
 	bool isClearingBackgroundAllowed() { return allow_clearing; };
 
+public slots:
+	//! Should be called when a file has started. 
+    /*! It's needed to know if the background has to be cleared or not. */
+	void playingStarted();
+	//! Should be called when a file has stopped.
+	void playingStopped();
+
 protected:
 	virtual void paintEvent ( QPaintEvent * e );
 
 private:
 	bool allow_clearing;
+	bool playing;
 };
 
 
@@ -86,11 +105,13 @@ class MplayerWindow : public Screen
     Q_OBJECT
 
 public:
-    MplayerWindow( QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
+    MplayerWindow( QWidget* parent = 0, Qt::WindowFlags f = 0);
     ~MplayerWindow();
     
 	void showLogo( bool b);
-	MplayerLayer * mplayerLayer();
+
+	MplayerLayer * videoLayer() { return mplayerlayer; };
+
 	void setResolution( int w, int h);
 	void setAspect( double asp);
 	void setMonitorAspect(double asp);
@@ -98,9 +119,6 @@ public:
 
 	void setColorKey(QColor c);
 
-	void allowClearingBackground(bool b);
-	bool isClearingBackgroundAllowed();
-
 	void setOffsetX( int );
 	int offsetX();
 
@@ -110,6 +128,9 @@ public:
 	void setZoom( double );
 	double zoom();
 
+	void allowVideoMovement(bool b) { allow_video_movement = b; };
+	bool isVideoMovementAllowed() { return allow_video_movement; };
+
 	virtual QSize sizeHint () const;
 	virtual QSize minimumSizeHint() const;
 
@@ -122,6 +143,9 @@ public slots:
 	void decZoom();
 
 protected:
+	virtual void retranslateStrings();
+	virtual void changeEvent ( QEvent * event ) ;
+
     virtual void resizeEvent( QResizeEvent * e);
     virtual void mouseReleaseEvent( QMouseEvent * e);
 	virtual void mouseDoubleClickEvent( QMouseEvent * e );
@@ -129,13 +153,13 @@ protected:
 	void moveLayer( int offset_x, int offset_y );
 
 protected slots:
-	virtual void languageChange();
 	virtual void translatePos(QPoint p );
 
 signals:
     void rightButtonReleased( QPoint p );
 	void doubleClicked();
 	void leftClicked();
+	void middleClicked();
 	void keyPressed(QKeyEvent * e);
 	void wheelUp();
 	void wheelDown();
@@ -156,6 +180,8 @@ protected:
 	// before zooming or moving
 	int orig_x, orig_y;
 	int orig_width, orig_height;
+
+	bool allow_video_movement;
 };
 
 
