@@ -18,29 +18,31 @@
 
 #include "playlist.h"
 
-#include <qtoolbar.h>
-#include <mytable.h>
+#include <QToolBar>
+#include <QFile>
+#include <QTextStream>
+#include <QDir>
+#include <QFileInfo>
+#include <QMessageBox>
+#include <QPushButton>
+#include <QRegExp>
+#include <QMenu>
+#include <QDateTime>
+#include <QSettings>
+#include <QInputDialog>
+#include <QToolButton>
+#include <QTimer>
+#include <QVBoxLayout>
+#include <QUrl>
+#include <QDragEnterEvent>
+#include <QDropEvent>
+#include <QHeaderView>
+#include <QTextCodec>
+#include <QApplication>
+
+#include "mytablewidget.h"
 #include "myaction.h"
-#include <qfile.h>
-#include <qtextstream.h>
-#include <qregexp.h>
 #include "filedialog.h"
-#include <qdir.h>
-#include <qfileinfo.h>
-#include <qmessagebox.h>
-#include <qpushbutton.h>
-#include <qpopupmenu.h>
-#include <qdragobject.h>
-#include <qurl.h>
-#include <qdatetime.h>
-#include <qsettings.h>
-#include <qinputdialog.h>
-#include <qtoolbutton.h>
-#include <qtooltip.h>
-#include <qtimer.h>
-
-#include <stdlib.h>
-
 #include "helper.h"
 #include "images.h"
 #include "preferences.h"
@@ -48,19 +50,20 @@
 #include "global.h"
 #include "core.h"
 #include "config.h"
+#include "extensions.h"
 
-#if KDE_SUPPORT
-#include <kurl.h>
-#endif
+#include <stdlib.h>
 
 
+#define DRAG_ITEMS 0
+
 #define COL_PLAY 0
 #define COL_NAME 1
 #define COL_TIME 2
 
 
-Playlist::Playlist( Core *c, QWidget * parent, const char * name, WFlags f)
-	: QMainWindow(parent, name, f)
+Playlist::Playlist( Core *c, QWidget * parent, Qt::WindowFlags f)
+	: QWidget(parent,f) 
 {
 	modified = false;
 
@@ -69,24 +72,31 @@ Playlist::Playlist( Core *c, QWidget * parent, const char * name, WFlags f)
     latest_dir = "";
 
 	createTable();
-	setCentralWidget( listView );
-
 	createActions();
 	createToolbar();
 
+	connect( core, SIGNAL(mediaFinished()), this, SLOT(playNext()) );
+	connect( core, SIGNAL(mediaLoaded()), this, SLOT(getMediaInfo()) );
+
+	QVBoxLayout *layout = new QVBoxLayout;
+	layout->addWidget( listView );
+	layout->addWidget( toolbar );
+	setLayout(layout);
+
     clear();
 
-	languageChange();
+	retranslateStrings();
 
 #if !DOCK_PLAYLIST
 	setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding );
 	adjustSize();
 #else
-	//setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );
-	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );
+	//setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Expanding );
+	//setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );
+	setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
 #endif
 
-	setAcceptDrops(TRUE);
+	setAcceptDrops(true);
 
 	// Random seed
 	QTime t;
@@ -113,136 +123,155 @@ void Playlist::setModified(bool mod) {
 }
 
 void Playlist::createTable() {
-	listView = new MyTable( this, "listView" );
-	listView->setSizeHint(QSize(100,100));
+	listView = new MyTableWidget( 0, COL_TIME + 1, this);
+	listView->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
+	listView->setSelectionBehavior(QAbstractItemView::SelectRows);
+	listView->setSelectionMode(QAbstractItemView::ExtendedSelection);
+	listView->setContextMenuPolicy( Qt::CustomContextMenu );
+	listView->setShowGrid(false);
+	listView->setSortingEnabled(false);
+	//listView->setAlternatingRowColors(true);
+	listView->horizontalHeader()->setResizeMode(QHeaderView::Interactive);
+	listView->horizontalHeader()->setResizeMode(COL_NAME, QHeaderView::Stretch);
+	/*
+	listView->horizontalHeader()->setResizeMode(COL_TIME, QHeaderView::ResizeToContents);
+	listView->horizontalHeader()->setResizeMode(COL_PLAY, QHeaderView::ResizeToContents);
+	*/
+	listView->setIconSize( Images::icon("ok_small").size() );
 
-	listView->setShowGrid( FALSE );
-	listView->setNumCols(COL_TIME + 1);
-	listView->setReadOnly(TRUE);
-	listView->setFocusStyle(QTable::FollowStyle);
-	listView->setSelectionMode(QTable::MultiRow);
-	//listView->verticalHeader()->hide();
-	listView->setSorting(FALSE);
+#if DRAG_ITEMS
+	listView->setSelectionMode(QAbstractItemView::SingleSelection);
+	listView->setDragEnabled(true);
+	listView->setAcceptDrops(true);
+	listView->setDropIndicatorShown(true);
+	listView->setDragDropMode(QAbstractItemView::InternalMove);
+#endif
 
-	connect( listView, SIGNAL(doubleClicked(int,int,int,const QPoint &)),
+	connect( listView, SIGNAL(cellActivated(int,int)),
              this, SLOT(itemDoubleClicked(int)) );
-
-	connect( core, SIGNAL(mediaFinished()), this, SLOT(playNext()) );
-	
-	connect( core, SIGNAL(mediaLoaded()),
-             this, SLOT(getMediaInfo()) );
 }
 
 void Playlist::createActions() {
-	openAct = new MyAction(this, "pl_open");
-	connect( openAct, SIGNAL(activated()), this, SLOT(load()) );
+	openAct = new MyAction(this, "pl_open", false);
+	connect( openAct, SIGNAL(triggered()), this, SLOT(load()) );
 
-	saveAct = new MyAction(this, "pl_save");
-	connect( saveAct, SIGNAL(activated()), this, SLOT(save()) );
+	saveAct = new MyAction(this, "pl_save", false);
+	connect( saveAct, SIGNAL(triggered()), this, SLOT(save()) );
 
-	playAct = new MyAction(this, "pl_play");
-	connect( playAct, SIGNAL(activated()), this, SLOT(playCurrent()) );
+	playAct = new MyAction(this, "pl_play", false);
+	connect( playAct, SIGNAL(triggered()), this, SLOT(playCurrent()) );
 
-	nextAct = new MyAction(Qt::Key_N /*Qt::Key_Greater*/, this, "pl_next");
-	connect( nextAct, SIGNAL(activated()), this, SLOT(playNext()) );
+	nextAct = new MyAction(Qt::Key_N /*Qt::Key_Greater*/, this, "pl_next", false);
+	connect( nextAct, SIGNAL(triggered()), this, SLOT(playNext()) );
 
-	prevAct = new MyAction(Qt::Key_P /*Qt::Key_Less*/, this, "pl_prev");
-	connect( prevAct, SIGNAL(activated()), this, SLOT(playPrev()) );
+	prevAct = new MyAction(Qt::Key_P /*Qt::Key_Less*/, this, "pl_prev", false);
+	connect( prevAct, SIGNAL(triggered()), this, SLOT(playPrev()) );
 
-	moveUpAct = new MyAction(this, "pl_move_up");
-	connect( moveUpAct, SIGNAL(activated()), this, SLOT(upItem()) );
+	moveUpAct = new MyAction(this, "pl_move_up", false);
+	connect( moveUpAct, SIGNAL(triggered()), this, SLOT(upItem()) );
 
-	moveDownAct = new MyAction(this, "pl_move_down");
-	connect( moveDownAct, SIGNAL(activated()), this, SLOT(downItem()) );
+	moveDownAct = new MyAction(this, "pl_move_down", false);
+	connect( moveDownAct, SIGNAL(triggered()), this, SLOT(downItem()) );
 
-	repeatAct = new MyAction(this, "pl_repeat");
-	repeatAct->setToggleAction(true);
+	repeatAct = new MyAction(this, "pl_repeat", false);
+	repeatAct->setCheckable(true);
 
-	shuffleAct = new MyAction(this, "pl_shuffle");
-	shuffleAct->setToggleAction(true);
+	shuffleAct = new MyAction(this, "pl_shuffle", false);
+	shuffleAct->setCheckable(true);
 
 	// Add actions
-	addCurrentAct = new MyAction(this, "pl_add_current");
-	connect( addCurrentAct, SIGNAL(activated()), this, SLOT(addCurrentFile()) );
+	addCurrentAct = new MyAction(this, "pl_add_current", false);
+	connect( addCurrentAct, SIGNAL(triggered()), this, SLOT(addCurrentFile()) );
 
-	addFilesAct = new MyAction(this, "pl_add_files");
-	connect( addFilesAct, SIGNAL(activated()), this, SLOT(addFiles()) );
+	addFilesAct = new MyAction(this, "pl_add_files", false);
+	connect( addFilesAct, SIGNAL(triggered()), this, SLOT(addFiles()) );
 
-	addDirectoryAct = new MyAction(this, "pl_add_directory");
-	connect( addDirectoryAct, SIGNAL(activated()), this, SLOT(addDirectory()) );
+	addDirectoryAct = new MyAction(this, "pl_add_directory", false);
+	connect( addDirectoryAct, SIGNAL(triggered()), this, SLOT(addDirectory()) );
 
 	// Remove actions
-	removeSelectedAct = new MyAction(this, "pl_remove_selected");
-	connect( removeSelectedAct, SIGNAL(activated()), this, SLOT(removeSelected()) );
+	removeSelectedAct = new MyAction(this, "pl_remove_selected", false);
+	connect( removeSelectedAct, SIGNAL(triggered()), this, SLOT(removeSelected()) );
 
-	removeAllAct = new MyAction(this, "pl_remove_all");
-	connect( removeAllAct, SIGNAL(activated()), this, SLOT(removeAll()) );
+	removeAllAct = new MyAction(this, "pl_remove_all", false);
+	connect( removeAllAct, SIGNAL(triggered()), this, SLOT(removeAll()) );
 
 	// Edit
-	editAct = new MyAction(this, "pl_edit");
-	connect( editAct, SIGNAL(activated()), this, SLOT(editCurrentItem()) );
+	editAct = new MyAction(this, "pl_edit", false);
+	connect( editAct, SIGNAL(triggered()), this, SLOT(editCurrentItem()) );
 }
 
 void Playlist::createToolbar() {
-	toolbar = new QToolBar(this, "toolbar");
-	toolbar->setResizeEnabled(false);
-	toolbar->setMovingEnabled(false);
-	addDockWindow(toolbar, Qt::DockBottom );
+	toolbar = new QToolBar(this);
+	toolbar->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
 
-	openAct->addTo( toolbar );
-	saveAct->addTo( toolbar );
+	toolbar->addAction(openAct);
+	toolbar->addAction(saveAct);;
 	toolbar->addSeparator();
 
-	add_menu = new QPopupMenu( this, "add_menu" );
-	addCurrentAct->addTo( add_menu );
-	addFilesAct->addTo( add_menu );
-	addDirectoryAct->addTo( add_menu );
+	add_menu = new QMenu( this );
+	add_menu->addAction(addCurrentAct);
+	add_menu->addAction(addFilesAct );
+	add_menu->addAction(addDirectoryAct);
 
-	add_button = new QToolButton( toolbar, "add_button" );
-	add_button->setPopup( add_menu );
-	add_button->setPopupDelay(1);
+	add_button = new QToolButton( this );
+	add_button->setMenu( add_menu );
+	add_button->setPopupMode(QToolButton::InstantPopup);
 
-	remove_menu = new QPopupMenu( this, "remove_menu" );
-	removeSelectedAct->addTo( remove_menu );
-	removeAllAct->addTo( remove_menu );
+	remove_menu = new QMenu( this );
+	remove_menu->addAction(removeSelectedAct);
+	remove_menu->addAction(removeAllAct);
 
-	remove_button = new QToolButton( toolbar, "remove_button" );
-	remove_button->setPopup( remove_menu );
-	remove_button->setPopupDelay(1);
+	remove_button = new QToolButton( this );
+	remove_button->setMenu( remove_menu );
+	remove_button->setPopupMode(QToolButton::InstantPopup);
+
+	toolbar->addWidget(add_button);
+	toolbar->addWidget(remove_button);
 
 	toolbar->addSeparator();
-	playAct->addTo( toolbar );
+	toolbar->addAction(playAct);
 	toolbar->addSeparator();
-	prevAct->addTo( toolbar );
-	nextAct->addTo( toolbar );
+	toolbar->addAction(prevAct);
+	toolbar->addAction(nextAct);
 	toolbar->addSeparator();
-	repeatAct->addTo( toolbar );
-	shuffleAct->addTo( toolbar );
+	toolbar->addAction(repeatAct);
+	toolbar->addAction(shuffleAct);
 	toolbar->addSeparator();
-	moveUpAct->addTo( toolbar );
-	moveDownAct->addTo( toolbar );
+	toolbar->addAction(moveUpAct);
+	toolbar->addAction(moveDownAct);
 
 	// Popup menu
-	popup = new QPopupMenu(this, "popup");
-	playAct->addTo( popup );
-	removeSelectedAct->addTo( popup );
-	editAct->addTo( popup );
-	connect( listView, SIGNAL(contextMenuRequested(int,int,const QPoint &)),
-             this, SLOT(showPopup(int,int,const QPoint &)) );
+	popup = new QMenu(this);
+	popup->addAction(playAct);
+	popup->addAction(removeSelectedAct);
+	popup->addAction(editAct);
+
+	connect( listView, SIGNAL(customContextMenuRequested(const QPoint &)),
+             this, SLOT(showPopup(const QPoint &)) );
 }
 
-void Playlist::languageChange() {
-	listView->horizontalHeader()->setLabel( COL_PLAY, "   " );
-	listView->horizontalHeader()->setLabel( COL_NAME, tr("Name") );
-	listView->horizontalHeader()->setLabel( COL_TIME, tr("Length") );
+void Playlist::retranslateStrings() {
+	listView->setHorizontalHeaderLabels( QStringList() << "   " <<
+        tr("Name") << tr("Length") );
 
 	openAct->change( Images::icon("open"), tr("&Load") );
 	saveAct->change( Images::icon("save"), tr("&Save") );
 
-	playAct->change( Images::icon("play"), tr("&Play") );
+	playAct->change( tr("&Play") );
 
-	nextAct->change( Images::icon("next"), tr("&Next") );
-	prevAct->change( Images::icon("previous"), tr("Pre&vious") );
+	nextAct->change( tr("&Next") );
+	prevAct->change( tr("Pre&vious") );
+
+	if (qApp->isLeftToRight()) {
+		playAct->setIcon( Images::icon("play") );
+		nextAct->setIcon( Images::icon("next") );
+		prevAct->setIcon( Images::icon("previous") );
+	} else {
+		playAct->setIcon( Images::flippedIcon("play") );
+		nextAct->setIcon( Images::flippedIcon("next") );
+		prevAct->setIcon( Images::flippedIcon("previous") );
+	}
 
 	moveUpAct->change( Images::icon("up"), tr("Move &up") );
 	moveDownAct->change( Images::icon("down"), tr("Move &down") );
@@ -263,14 +292,14 @@ void Playlist::languageChange() {
 	editAct->change( tr("&Edit") );
 
 	// Tool buttons
-	add_button->setIconSet( Images::icon("plus") );
-	QToolTip::add( add_button, tr("Add...") );
-	remove_button->setIconSet( Images::icon("minus") );
-	QToolTip::add( remove_button, tr("Remove...") );
+	add_button->setIcon( Images::icon("plus") );
+	add_button->setToolTip( tr("Add...") );
+	remove_button->setIcon( Images::icon("minus") );
+	remove_button->setToolTip( tr("Remove...") );
 
 	// Icon
-	setIcon( Images::icon("logo", 64) );
-	setCaption( tr( "SMPlayer - Playlist" ) );
+	setWindowIcon( Images::icon("logo", 64) );
+	setWindowTitle( tr( "SMPlayer - Playlist" ) );
 }
 
 void Playlist::list() {
@@ -279,42 +308,37 @@ void Playlist::list() {
 	PlaylistItemList::iterator it;
 	for ( it = pl.begin(); it != pl.end(); ++it ) {
 		qDebug( "filename: '%s', name: '%s' duration: %f",
-               (*it).filename().utf8().data(), (*it).name().utf8().data(),
+               (*it).filename().toUtf8().data(), (*it).name().toUtf8().data(),
                (*it).duration() );
 	}
 }
 
 void Playlist::updateView() {
-	listView->setNumRows( pl.count() );
+	listView->setRowCount( pl.count() );
 
-	int n=0;
-	QString number;
+	//QString number;
 	QString name;
 	QString time;
 
-	PlaylistItemList::iterator it;
-
-	for ( it = pl.begin(); it != pl.end(); ++it ) {
-		number.sprintf("%03d", n +1 );
-		name = (*it).name();
-		if (name.isEmpty()) name = (*it).filename();
-		time = Helper::formatTime( (int) (*it).duration() );
+	for (int n=0; n < pl.count(); n++) {
+		name = pl[n].name();
+		if (name.isEmpty()) name = pl[n].filename();
+		time = Helper::formatTime( (int) pl[n].duration() );
 		
 		//listView->setText(n, COL_POS, number);
+		qDebug("name: '%s'", name.toUtf8().data());
 		listView->setText(n, COL_NAME, name);
 		listView->setText(n, COL_TIME, time);
 
-		if ((*it).played()) {
-			listView->setPixmap(n, COL_PLAY, Images::icon("ok_small") );
+		if (pl[n].played()) {
+			listView->setIcon(n, COL_PLAY, Images::icon("ok_small") );
 		} else {
-			listView->setPixmap(n, COL_PLAY, QPixmap() );
+			listView->setIcon(n, COL_PLAY, QPixmap() );
 		}
-
-		n++;
 	}
-
-	for (n=0; n <= COL_TIME; n++) 
-		listView->adjustColumn(n);
+	//listView->resizeColumnsToContents();
+	listView->resizeColumnToContents(COL_PLAY);
+	listView->resizeColumnToContents(COL_TIME);
 
 	setCurrentItem(current_item);
 
@@ -322,6 +346,13 @@ void Playlist::updateView() {
 }
 
 void Playlist::setCurrentItem(int current) {
+	QIcon play_icon;
+	if (qApp->isLeftToRight()) {
+		play_icon = Images::icon("play");
+	} else {
+		play_icon = Images::flippedIcon("play");
+	}
+
 	int old_current = current_item;
 	current_item = current;
 
@@ -329,24 +360,26 @@ void Playlist::setCurrentItem(int current) {
 		pl[current_item].setPlayed(TRUE);
 	}
 
-	if ( (old_current >= 0) && (old_current < listView->numRows()) ) {
-		listView->setPixmap(old_current, COL_PLAY, QPixmap() );
+	if ( (old_current >= 0) && (old_current < listView->rowCount()) ) {
+		listView->setIcon(old_current, COL_PLAY, QPixmap() );
 	}
 
-	if ( (current_item >= 0) && (current_item < listView->numRows()) ) {
-		listView->setPixmap(current_item, COL_PLAY, Images::icon("play") );
+	if ( (current_item >= 0) && (current_item < listView->rowCount()) ) {
+		listView->setIcon(current_item, COL_PLAY, play_icon );
 	}
 	//if (current_item >= 0) listView->selectRow(current_item);
 	if (current_item >= 0) {
 		listView->clearSelection();
 		listView->setCurrentCell( current_item, 0);
-		listView->selectRow(current_item);
 	}
 }
 
 void Playlist::clear() {
 	pl.clear();
-	listView->setNumRows(0);
+
+	listView->clearContents();
+	listView->setRowCount(0);
+
 	setCurrentItem(0);
 
 	setModified( false );
@@ -361,7 +394,7 @@ bool Playlist::isEmpty() {
 }
 
 void Playlist::addItem(QString filename, QString name, double duration) {
-	qDebug("Playlist::addItem: '%s'", filename.utf8().data());
+	qDebug("Playlist::addItem: '%s'", filename.toUtf8().data());
 
 	#ifdef Q_OS_WIN
 	filename = Helper::changeSlashes(filename);
@@ -398,14 +431,14 @@ void Playlist::addItem(QString filename, QString name, double duration) {
 void Playlist::load_m3u(QString file) {
 	qDebug("Playlist::load_m3u");
 
-	bool utf8 = (QFileInfo(file).extension(FALSE).lower() == "m3u8");
+	bool utf8 = (QFileInfo(file).suffix().toLower() == "m3u8");
 
 	QRegExp m3u_id("^#EXTM3U|^#M3U");
 	QRegExp info("^#EXTINF:(.*),(.*)");
 
     QFile f( file );
-    if ( f.open( IO_ReadOnly ) ) {
-		playlist_path = QFileInfo(file).dirPath();
+    if ( f.open( QIODevice::ReadOnly ) ) {
+		playlist_path = QFileInfo(file).path();
 
 		clear();
 		QString filename="";
@@ -415,23 +448,23 @@ void Playlist::load_m3u(QString file) {
         QTextStream stream( &f );
 
 		if (utf8)
-			stream.setEncoding(QTextStream::UnicodeUTF8);
+			stream.setCodec("UTF-8");
 		else
-			stream.setEncoding(QTextStream::Locale);
+			stream.setCodec(QTextCodec::codecForLocale());
 
         QString line;
         while ( !stream.atEnd() ) {
             line = stream.readLine(); // line of text excluding '\n'
-            qDebug( " * line: '%s'", line.utf8().data() );
-			if (m3u_id.search(line)!=-1) {
+            qDebug( " * line: '%s'", line.toUtf8().data() );
+			if (m3u_id.indexIn(line)!=-1) {
 				//#EXTM3U
 				// Ignore line
 			}
 			else
-			if (info.search(line)!=-1) {
+			if (info.indexIn(line)!=-1) {
 				duration = info.cap(1).toDouble();
 				name = info.cap(2);
-				qDebug(" * name: '%s', duration: %f", name.utf8().data(), duration );
+				qDebug(" * name: '%s', duration: %f", name.toUtf8().data(), duration );
 			} 
 			else
 			if (line.startsWith("#")) {
@@ -441,7 +474,7 @@ void Playlist::load_m3u(QString file) {
 				filename = line;
 				QFileInfo fi(filename);
 				if (fi.exists()) {
-					filename = fi.absFilePath();
+					filename = fi.absoluteFilePath();
 				}
 				if (!fi.exists()) {
 					if (QFileInfo( playlist_path + "/" + filename).exists() ) {
@@ -464,32 +497,32 @@ void Playlist::load_m3u(QString file) {
 }
 
 bool Playlist::save_m3u(QString file) {
-	qDebug("Playlist::save_m3u: '%s'", file.utf8().data());
+	qDebug("Playlist::save_m3u: '%s'", file.toUtf8().data());
 
-	QString dir_path = QFileInfo(file).dirPath();
+	QString dir_path = QFileInfo(file).path();
 	if (!dir_path.endsWith("/")) dir_path += "/";
 
 	#ifdef Q_OS_WIN
 	dir_path = Helper::changeSlashes(dir_path);
 	#endif
 
-	qDebug(" * dirPath: '%s'", dir_path.utf8().data());
+	qDebug(" * dirPath: '%s'", dir_path.toUtf8().data());
 
-	bool utf8 = (QFileInfo(file).extension(FALSE).lower() == "m3u8");
+	bool utf8 = (QFileInfo(file).suffix().toLower() == "m3u8");
 
 	QFile f( file );
-    if ( f.open( IO_WriteOnly ) ) {
+    if ( f.open( QIODevice::WriteOnly ) ) {
         QTextStream stream( &f );
 
 		if (utf8) 
-			stream.setEncoding(QTextStream::UnicodeUTF8);
+			stream.setCodec("UTF-8");
 		else
-			stream.setEncoding(QTextStream::Locale);
+			stream.setCodec(QTextCodec::codecForLocale());
 
 		QString filename;
 
 		stream << "#EXTM3U" << "\n";
-		stream << "# Playlist created by SMPlayer " << VERSION << " \n";
+		stream << "# Playlist created by SMPlayer " << smplayerVersion() << " \n";
 
 		PlaylistItemList::iterator it;
 		for ( it = pl.begin(); it != pl.end(); ++it ) {
@@ -517,31 +550,29 @@ bool Playlist::save_m3u(QString file) {
 
 void Playlist::load() {
 	if (maybeSave()) {
+		Extensions e;
 		QString s = MyFileDialog::getOpenFileName(
+                    this, tr("Choose a file"), 
                     lastDir(),
-                    tr("Playlists") +" (*.m3u *.m3u8)",
-                    this,
-                    "open file dialog",
-                    tr("Choose a file") );
+                    tr("Playlists") + e.playlist().forFilter());
 
 		if (!s.isEmpty()) {
-			latest_dir = QFileInfo(s).dirPath(TRUE);
+			latest_dir = QFileInfo(s).absolutePath();
 			load_m3u(s);
 		}
 	}
 }
 
 bool Playlist::save() {
+	Extensions e;
 	QString s = MyFileDialog::getSaveFileName(
+                    this, tr("Choose a filename"), 
                     lastDir(),
-                    tr("Playlists") +" (*.m3u *.m3u8)",
-                    this,
-                    "save file dialog",
-                    tr("Choose a filename") );
+                    tr("Playlists") + e.playlist().forFilter());
 
 	if (!s.isEmpty()) {
 		// If filename has no extension, add it
-		if (QFileInfo(s).extension().isEmpty()) {
+		if (QFileInfo(s).suffix().isEmpty()) {
 			s = s + ".m3u";
 		}
 		if (QFileInfo(s).exists()) {
@@ -556,7 +587,7 @@ bool Playlist::save() {
             	return false;
 			}
 		}
-		latest_dir = QFileInfo(s).dirPath(TRUE);
+		latest_dir = QFileInfo(s).absolutePath();
 		return save_m3u(s);
 	} else {
 		return false;
@@ -592,17 +623,18 @@ void Playlist::itemDoubleClicked(int row) {
 	playItem(row);
 }
 
-void Playlist::showPopup(int row, int col, const QPoint & pos) {
-	qDebug("Playlist::showPopup: row: %d col: %d", row, col );
+void Playlist::showPopup(const QPoint & pos) {
+	qDebug("Playlist::showPopup: x: %d y: %d", pos.x(), pos.y() );
+
 	if (!popup->isVisible()) {
-		popup->move( pos );
+		popup->move( listView->viewport()->mapToGlobal(pos) );
 		popup->show();
 	}
 }
 
 void Playlist::startPlay() {
 	// Start to play
-	if ( shuffleAct->isOn() ) 
+	if ( shuffleAct->isChecked() ) 
 		playItem( chooseRandomItem() );
 	else
 		playItem(0);
@@ -617,7 +649,7 @@ void Playlist::playItem( int n ) {
 		return;
 	}
 
-	qDebug(" playlist_path: '%s'", playlist_path.utf8().data() );
+	qDebug(" playlist_path: '%s'", playlist_path.toUtf8().data() );
 
 	QString filename = pl[n].filename();
 	QString filename_with_path = playlist_path + "/" + filename;
@@ -633,19 +665,19 @@ void Playlist::playItem( int n ) {
 void Playlist::playNext() {
 	qDebug("Playlist::playNext");
 
-	if (shuffleAct->isOn()) {
+	if (shuffleAct->isChecked()) {
 		// Shuffle
 		int chosen_item = chooseRandomItem();
 		if (chosen_item == -1) {
 			clearPlayedTag();
-			if (repeatAct->isOn()) chosen_item = chooseRandomItem();
+			if (repeatAct->isChecked()) chosen_item = chooseRandomItem();
 		}
 		playItem( chosen_item );
 	} else {
 		bool finished_list = (current_item+1 >= pl.count());
 		if (finished_list) clearPlayedTag();
 
-		if ( (repeatAct->isOn()) && (finished_list) ) {
+		if ( (repeatAct->isChecked()) && (finished_list) ) {
 			playItem(0);
 		} else {
 			playItem( current_item+1 );
@@ -717,11 +749,9 @@ void Playlist::addCurrentFile() {
 
 void Playlist::addFiles() {
 	QStringList files = MyFileDialog::getOpenFileNames(
-                            tr("All files") +" (*.*)",
+                            this, tr("Select one or more files to open"), 
                             lastDir(),
-                            this,
-                            "open files dialog",
-                            tr("Select one or more files to open") );
+                            tr("All files") +" (*.*)" );
 
 	if (files.count()!=0) addFiles(files);  
 }
@@ -739,16 +769,17 @@ void Playlist::addFiles(QStringList files) {
     }
 	updateView();
 
-	qDebug( " * latest_dir: '%s'", latest_dir.utf8().data() );
+	qDebug( " * latest_dir: '%s'", latest_dir.toUtf8().data() );
+}
+
+void Playlist::addFile(QString file) {
+	addFiles( QStringList() << file );
 }
 
 void Playlist::addDirectory() {
 	QString s = MyFileDialog::getExistingDirectory(
-                    lastDir(),
-                    this,
-                    "get existing directory",
-                    tr("Choose a directory"),
-                    TRUE );
+                    this, tr("Choose a directory"),
+                    lastDir() );
 
 	if (!s.isEmpty()) {
 		addDirectory(s);
@@ -779,8 +810,8 @@ void Playlist::removeSelected() {
 
 	int first_selected = -1;
 
-	for (int n=0; n < listView->numRows(); n++) {
-		if (listView->isRowSelected(n)) {
+	for (int n=0; n < listView->rowCount(); n++) {
+		if (listView->isSelected(n, 0)) {
 			qDebug(" row %d selected", n);
 			pl[n].setMarkForDeletion(TRUE);
 			if (first_selected == -1) first_selected = n;
@@ -790,8 +821,8 @@ void Playlist::removeSelected() {
 	PlaylistItemList::iterator it;
 	for ( it = pl.begin(); it != pl.end(); ++it ) {
 		if ( (*it).markedForDeletion() ) {
-			qDebug("Remove '%s'", (*it).filename().utf8().data());
-			it = pl.remove(it);
+			qDebug("Remove '%s'", (*it).filename().toUtf8().data());
+			it = pl.erase(it);
 			it--;
 			setModified( true );
 		}
@@ -800,13 +831,13 @@ void Playlist::removeSelected() {
 	if (isEmpty()) setModified(false);
 	updateView();
 
-	if (first_selected >= listView->numRows()) 
-		first_selected = listView->numRows() - 1;
+	if (first_selected >= listView->rowCount()) 
+		first_selected = listView->rowCount() - 1;
 
-	if ( ( first_selected > -1) && ( first_selected < listView->numRows() ) ) {
+	if ( ( first_selected > -1) && ( first_selected < listView->rowCount() ) ) {
 		listView->clearSelection();
 		listView->setCurrentCell( first_selected, 0);
-		listView->selectRow( first_selected );
+		//listView->selectRow( first_selected );
 	}
 }
 
@@ -829,7 +860,7 @@ void Playlist::clearPlayedTag() {
 
 int Playlist::chooseRandomItem() {
 	qDebug( "Playlist::chooseRandomItem");
-	QValueList <int> fi; //List of not played items (free items)
+	QList <int> fi; //List of not played items (free items)
 
 	int n=0;
 	PlaylistItemList::iterator it;
@@ -904,11 +935,11 @@ void Playlist::editItem(int item) {
 	if (current_name.isEmpty()) current_name = pl[item].filename();
 
 	bool ok;
-	QString text = QInputDialog::getText(
+	QString text = QInputDialog::getText( this,
             tr("Edit name"), 
             tr("Type the name that will be displayed in the playlist for this file:"), 
             QLineEdit::Normal,
-            current_name, &ok, this );
+            current_name, &ok );
     if ( ok && !text.isEmpty() ) {
         // user entered something and pressed OK
 		pl[item].setName(text);
@@ -924,76 +955,56 @@ void Playlist::editItem(int item) {
 // Drag&drop
 void Playlist::dragEnterEvent( QDragEnterEvent *e ) {
 	qDebug("Playlist::dragEnterEvent");
-	 #if QT_VERSION < 0x040000
-	e->accept( (QUriDrag::canDecode(e) || QTextDrag::canDecode(e)) );
-	#else
-	e->accept(QUriDrag::canDecode(e));
-	#endif
-}
 
+	if (e->mimeData()->hasUrls()) {
+		e->acceptProposedAction();
+	}
+}
 
 void Playlist::dropEvent( QDropEvent *e ) {
 	qDebug("Playlist::dropEvent");
 
 	QStringList files;
 
-	if (QUriDrag::canDecode(e)) {
-		QStrList l;
-		QUriDrag::decode(e, l);
-
+	if (e->mimeData()->hasUrls()) {
+		QList <QUrl> l = e->mimeData()->urls();
 		QString s;
-        for ( unsigned int i= 0; i < l.count(); ++i ) {
-			s = l.at(i);
-			qDebug(" * '%s'", s.utf8().data() );
-#if KDE_SUPPORT
-			KURL u(s);
-#else
-			QUrl u(s);
-#endif
-			qDebug(" * protocol: '%s'", u.protocol().utf8().data());
-			qDebug(" * path: '%s'", u.path().utf8().data());
-			//qDebug(" filename:='%s'", u.fileName().utf8().data());
-			if (u.protocol()=="file") {
-				s = u.path();
+		for (int n=0; n < l.count(); n++) {
+			if (l[n].isValid()) {
+				qDebug("Playlist::dropEvent: scheme: '%s'", l[n].scheme().toUtf8().data());
+				if (l[n].scheme() == "file") 
+					s = l[n].toLocalFile();
+				else
+					s = l[n].toString();
+				/*
+				qDebug(" * '%s'", l[n].toString().toUtf8().data());
+				qDebug(" * '%s'", l[n].toLocalFile().toUtf8().data());
+				*/
+				qDebug("Playlist::dropEvent: file: '%s'", s.toUtf8().data());
+				files.append(s);
 			}
-			files.append( s );
-
 		}
 	}
 
-	#if QT_VERSION < 0x040000
-	else
-	if (QTextDrag::canDecode(e)) {
-		QString s;
-		if (QTextDrag::decode(e, s)) {
-			qDebug(" * '%s'", s.utf8().data() );
-			files.append( s );
-		}
-	}
-	#endif
 
 	QStringList only_files;
 	for (int n = 0; n < files.count(); n++) {
 		if ( QFileInfo( files[n] ).isDir() ) {
 			addDirectory( files[n] );
 		} else {
-			//addFiles( files[n] );
-			//addItem( files[n], "", 0 );
 			only_files.append( files[n] );
 		}
 	}
-	//updateView();
-	//addFiles(files);
 	addFiles( only_files );
 }
 
 
 void Playlist::hideEvent( QHideEvent * ) {
-	emit visibilityChanged();
+	emit visibilityChanged(false);
 }
 
 void Playlist::showEvent( QShowEvent * ) {
-	emit visibilityChanged();
+	emit visibilityChanged(true);
 }
 
 void Playlist::closeEvent( QCloseEvent * e )  {
@@ -1014,27 +1025,27 @@ void Playlist::saveSettings() {
 
 	set->beginGroup( "playlist");
 
-	set->writeEntry( "repeat", repeatAct->isOn() );
-	set->writeEntry( "shuffle", shuffleAct->isOn() );
-#if !DOCK_PLAYLIST
-	set->writeEntry( "window_width", size().width() );
-	set->writeEntry( "window_height", size().height() );
-#endif
-	set->writeEntry( "latest_dir", latest_dir );
+	set->setValue( "repeat", repeatAct->isChecked() );
+	set->setValue( "shuffle", shuffleAct->isChecked() );
+//#if !DOCK_PLAYLIST
+	set->setValue( "window_width", size().width() );
+	set->setValue( "window_height", size().height() );
+//#endif
+	set->setValue( "latest_dir", latest_dir );
 
 	set->endGroup();
 
 	//Save current list
 	set->beginGroup( "playlist_contents");
 
-	set->writeEntry( "count", (int) pl.count() );
+	set->setValue( "count", (int) pl.count() );
 	for ( int n=0; n < pl.count(); n++ ) {
-		set->writeEntry( QString("item_%1_filename").arg(n), pl[n].filename() );
-		set->writeEntry( QString("item_%1_duration").arg(n), pl[n].duration() );
-		set->writeEntry( QString("item_%1_name").arg(n), pl[n].name() );
+		set->setValue( QString("item_%1_filename").arg(n), pl[n].filename() );
+		set->setValue( QString("item_%1_duration").arg(n), pl[n].duration() );
+		set->setValue( QString("item_%1_name").arg(n), pl[n].name() );
 	}
-	set->writeEntry( "current_item", current_item );
-	set->writeEntry( "modified", modified );
+	set->setValue( "current_item", current_item );
+	set->setValue( "modified", modified );
 
 	set->endGroup();
 }
@@ -1046,34 +1057,34 @@ void Playlist::loadSettings() {
 
 	set->beginGroup( "playlist");
 
-	repeatAct->setOn( set->readBoolEntry( "repeat", repeatAct->isOn() ) );
-	shuffleAct->setOn( set->readBoolEntry( "shuffle", shuffleAct->isOn() ) );
+	repeatAct->setChecked( set->value( "repeat", repeatAct->isChecked() ).toBool() );
+	shuffleAct->setChecked( set->value( "shuffle", shuffleAct->isChecked() ).toBool() );
 
-#if !DOCK_PLAYLIST
+//#if !DOCK_PLAYLIST
 	QSize s;
-	s.setWidth( set->readNumEntry( "window_width", size().width() ) );
-	s.setHeight( set->readNumEntry( "window_height", size().height() ) );
+	s.setWidth( set->value( "window_width", size().width() ).toInt() );
+	s.setHeight( set->value( "window_height", size().height() ).toInt() );
 	resize( s );
-#endif
+//#endif
 
-	latest_dir = set->readEntry( "latest_dir", latest_dir );
+	latest_dir = set->value( "latest_dir", latest_dir ).toString();
 
 	set->endGroup();
 
 	//Load latest list
 	set->beginGroup( "playlist_contents");
 
-	int count = set->readNumEntry( "count", 0 );
+	int count = set->value( "count", 0 ).toInt();
 	QString filename, name;
-	int duration;
+	double duration;
 	for ( int n=0; n < count; n++ ) {
-		filename = set->readEntry( QString("item_%1_filename").arg(n), "" );
-		duration = set->readNumEntry( QString("item_%1_duration").arg(n), -1 );
-		name = set->readEntry( QString("item_%1_name").arg(n), "" );
+		filename = set->value( QString("item_%1_filename").arg(n), "" ).toString();
+		duration = set->value( QString("item_%1_duration").arg(n), -1 ).toDouble();
+		name = set->value( QString("item_%1_name").arg(n), "" ).toString();
 		addItem( filename, name, duration );
 	}
-	setCurrentItem( set->readNumEntry( "current_item", -1 ) );
-	setModified( set->readBoolEntry( "modified", false ) );
+	setCurrentItem( set->value( "current_item", -1 ).toInt() );
+	setModified( set->value( "modified", false ).toBool() );
 	updateView();
 
 	set->endGroup();
@@ -1085,3 +1096,13 @@ QString Playlist::lastDir() {
 	return last_dir;
 }
 
+// Language change stuff
+void Playlist::changeEvent(QEvent *e) {
+	if (e->type() == QEvent::LanguageChange) {
+		retranslateStrings();
+	} else {
+		QWidget::changeEvent(e);
+	}
+}
+
+#include "moc_playlist.cpp"
