@@ -17,12 +17,12 @@
 */
 
 
-#ifndef _PLAYLIST2_H_
-#define _PLAYLIST2_H_
+#ifndef _PLAYLIST_H_
+#define _PLAYLIST_H_
 
-#include <qvaluelist.h>
-#include <qstringlist.h>
-#include <qmainwindow.h>
+#include <QList>
+#include <QStringList>
+#include <QWidget>
 
 class PlaylistItem {
 
@@ -52,22 +52,21 @@ private:
 	bool _played, _deleted;
 };
 
-
-class MyTable;
+class MyTableWidget;
 class QToolBar;
 class MyAction;
 class Core;
-class QPopupMenu;
+class QMenu;
 class QSettings;
 class QToolButton;
 class QTimer;
 
-class Playlist : public QMainWindow
+class Playlist : public QWidget
 {
 	Q_OBJECT
 
 public:
-	Playlist( Core *c, QWidget * parent = 0, const char * name = 0, WFlags f = WType_TopLevel );
+	Playlist( Core *c, QWidget * parent = 0, Qt::WindowFlags f = Qt::Window );
 	~Playlist();
 
 	void clear();
@@ -96,6 +95,7 @@ public slots:
 	virtual void addFiles();
 	virtual void addDirectory();
 
+	virtual void addFile(QString file);
 	virtual void addFiles(QStringList files);
 	virtual void addDirectory(QString dir);
 
@@ -118,7 +118,7 @@ public:
 
 signals:
 	void playlistEnded();
-	void visibilityChanged();
+	void visibilityChanged(bool visible);
 	void modifiedChanged(bool);
 
 protected:
@@ -129,18 +129,10 @@ protected:
 	void swapItems(int item1, int item2 );
 	QString lastDir();
 
-	virtual void dragEnterEvent( QDragEnterEvent * ) ;
-	virtual void dropEvent ( QDropEvent * );
-	virtual void hideEvent ( QHideEvent * );
-	virtual void showEvent ( QShowEvent * );
-	virtual void closeEvent( QCloseEvent * e );
-
 protected slots:
-	virtual void languageChange();
-
 	virtual void playCurrent();
 	virtual void itemDoubleClicked(int row);
-	virtual void showPopup(int row, int col, const QPoint & pos);
+	virtual void showPopup(const QPoint & pos);
 	virtual void upItem();
 	virtual void downItem();
 	virtual void editCurrentItem();
@@ -157,7 +149,16 @@ protected:
 	void createToolbar();
 
 protected:
-	typedef QValueList <PlaylistItem> PlaylistItemList;
+	void retranslateStrings();
+	virtual void changeEvent ( QEvent * event ) ;
+	virtual void dragEnterEvent( QDragEnterEvent * ) ;
+	virtual void dropEvent ( QDropEvent * );
+	virtual void hideEvent ( QHideEvent * );
+	virtual void showEvent ( QShowEvent * );
+	virtual void closeEvent( QCloseEvent * e );
+
+protected:
+	typedef QList <PlaylistItem> PlaylistItemList;
 	PlaylistItemList pl;
 	int current_item;
 
@@ -165,11 +166,11 @@ protected:
 	QString latest_dir;
 
 	Core * core;
-	QPopupMenu * add_menu;
-	QPopupMenu * remove_menu;
-	QPopupMenu * popup;
+	QMenu * add_menu;
+	QMenu * remove_menu;
+	QMenu * popup;
 
-	MyTable * listView;
+	MyTableWidget * listView;
 
 	QToolBar * toolbar;
 	QToolButton * add_button;
