@@ -19,15 +19,15 @@
 #ifndef _RECENTS_H_
 #define _RECENTS_H_
 
-#include <qstringlist.h>
-#include <qobject.h>
+#include <QStringList>
+#include <QObject>
 
 class Recents : public QObject
 {
 	Q_OBJECT
 
 public:
-	Recents(QObject* parent = 0, const char* name = 0);
+	Recents(QObject* parent = 0);
 	~Recents();
 
 	void add(QString s);
