@@ -19,7 +19,7 @@
 
 #include "subtracks.h"
 #include "mediasettings.h"
-#include <qregexp.h>
+#include <QRegExp>
 
 SubTracks::SubTracks() {
 	index = 0;
@@ -44,15 +44,15 @@ void SubTracks::add( SubData::Type t, int ID ) {
 void SubTracks::list() {
 	for (unsigned int n=0; n < subs.count(); n++) {
 		qDebug("SubTracks::list: item %d: type: %d ID: %d lang: '%s' name: '%s' filename: '%s'",
-               n, subs[n].type(), subs[n].ID(), subs[n].lang().utf8().data(),
-               subs[n].name().utf8().data(), subs[n].filename().utf8().data() );
+               n, subs[n].type(), subs[n].ID(), subs[n].lang().toUtf8().data(),
+               subs[n].name().toUtf8().data(), subs[n].filename().toUtf8().data() );
 	}
 }
 
 void SubTracks::listNames() {
 	for (unsigned int n=0; n < subs.count(); n++) {
 		qDebug("SubTracks::list: item %d: '%s'",
-               n, subs[n].displayName().utf8().data() );
+               n, subs[n].displayName().toUtf8().data() );
 	}
 }
 
@@ -60,16 +60,20 @@ int SubTracks::numItems() {
 	return subs.count();
 }
 
+bool SubTracks::existsItemAt(int n) {
+	return ((n > 0) && (n < numItems()));
+}
+
 int SubTracks::findLang(QString expr) {
-	qDebug( "SubTracks::findLang: '%s'", expr.utf8().data());
+	qDebug( "SubTracks::findLang: '%s'", expr.toUtf8().data());
 	QRegExp rx( expr );
 
 	int res_id = -1;
 
 	for (int n=0; n < numItems(); n++) {
 		qDebug("SubTracks::findLang: lang #%d '%s'", n, 
-                subs[n].lang().utf8().data());
-		if (rx.search( subs[n].lang() ) > -1) {
+                subs[n].lang().toUtf8().data());
+		if (rx.indexIn( subs[n].lang() ) > -1) {
 			qDebug("SubTracks::findLang: found preferred lang!");
 			res_id = n;
 			break;	
@@ -81,11 +85,14 @@ int SubTracks::findLang(QString expr) {
 
 // Return first subtitle or the user preferred (if found)
 // or none if there's no subtitles
-int SubTracks::selectOne(QString preferred_lang) {
+int SubTracks::selectOne(QString preferred_lang, int default_sub) {
 	int sub = MediaSettings::SubNone; 
 
 	if (numItems() > 0) {
 		sub = 0; // First subtitle
+		if (existsItemAt(default_sub)) {
+			sub = default_sub;
+		}
 
 		// Check if one of the subtitles is the user preferred.
 		if (!preferred_lang.isEmpty()) {
@@ -144,13 +151,13 @@ bool SubTracks::changeFilename( SubData::Type t, int ID, QString filename ) {
 }
 
 void SubTracks::process(QString text) {
-	qDebug("SubTracks::process: '%s'", text.utf8().data());
+	qDebug("SubTracks::process: '%s'", text.toUtf8().data());
 
 	QRegExp rx_subtitle("^ID_(SUBTITLE|FILE_SUB|VOBSUB)_ID=(\\d+)");
 	QRegExp rx_sid("^ID_(SID|VSID)_(\\d+)_(LANG|NAME)=(.*)");
 	QRegExp rx_subtitle_file("^ID_FILE_SUB_FILENAME=(.*)");
 
-	if (rx_subtitle.search(text) > -1) {
+	if (rx_subtitle.indexIn(text) > -1) {
 		int ID = rx_subtitle.cap(2).toInt();
 		QString type = rx_subtitle.cap(1);
 
@@ -168,7 +175,7 @@ void SubTracks::process(QString text) {
 		}	
 	}
 	else
-	if (rx_sid.search(text) > -1) {
+	if (rx_sid.indexIn(text) > -1) {
 		int ID = rx_sid.cap(2).toInt();
 		QString value = rx_sid.cap(4);
 		QString attr = rx_sid.cap(3);
@@ -187,7 +194,7 @@ void SubTracks::process(QString text) {
 		}	
 	}
 	else
-	if (rx_subtitle_file.search(text) > -1) {
+	if (rx_subtitle_file.indexIn(text) > -1) {
 		QString file = rx_subtitle_file.cap(1);
 		if ( subs.count() > 0 ) {
 			int last = subs.count() -1;
@@ -198,6 +205,7 @@ void SubTracks::process(QString text) {
 	}
 }
 
+/*
 void SubTracks::test() {
 	process("ID_SUBTITLE_ID=0");
 	process("ID_SID_0_NAME=Arabic");
@@ -217,3 +225,4 @@ void SubTracks::test() {
 	list();
 	listNames();
 }
+*/
