@@ -19,9 +19,8 @@
 #ifndef _TRANSLATOR_H_
 #define _TRANSLATOR_H_
 
-#include <qstring.h>
-
-class QTranslator;
+#include <QString>
+#include <QTranslator>
 
 class Translator
 {
@@ -30,13 +29,13 @@ public:
 	Translator();
 	~Translator();
 
-	QTranslator * qtranslator() { return trans; };
-
 	void load(QString locale);
 
 protected:
-	QTranslator * trans;
+	static bool loadCatalog(QTranslator & t, QString name, QString locale, QString dir);
 
+	QTranslator app_trans;
+	QTranslator qt_trans;
 };
 
 #endif
