@@ -19,7 +19,9 @@
 #ifndef _VERSION_H_
 #define _VERSION_H_
 
-#define VERSION "0.5.20"
+#include <QString>
+
+QString smplayerVersion();
 
 #endif
 
